@@ -30,9 +30,6 @@
 #if defined(__WIN32__)
 #if defined(HAVE_WINSOCK2_H)
 #include <winsock2.h>
-#if _MSC_VER < 1200
-#define HAVE_INT32 1
-#endif
 #endif
 #if defined(__cplusplus)
 #define HAVE_BOOL 1
@@ -80,3 +77,12 @@
 #define DOUBLE_ORDER_OPPOSITE
 #endif
 
+/*
+ * GCC before 3.0 does not support explicit branch optimization.
+ */
+#if !defined(__GNUC__) || (__GNUC__ < 3) || (__GNUC__ == 3 && __GNUC_MINOR__ == 0)
+#define __builtin_expect(a,b) (a)
+
+#include <stdlib.h>
+#define __builtin_trap() abort()
+#endif
