@@ -22,8 +22,7 @@
 /**/
 
 /* Function prototype for signal handlers */
-#define	EXCEPTIONPROTO							\
-	int sig, int code, struct sigcontext* ctx
+#define	EXCEPTIONPROTO SIGNAL_ARGS(sig, ctx)
 
 /* Get the first exception frame from a signal handler */
 #define	EXCEPTIONFRAME(f, c)						\
