@@ -19,8 +19,14 @@
 /**/
 /* Include common information. */
 /**/
-#include "m68k/netbsd1/jit.h"
+#include "m68k/jit.h"
 
+/*
+ * Include m68k_sync_icache prototypes
+ */
+#include <sys/types.h>
+#include <m68k/sync_icache.h>
+ 
 /**/
 /* Define where addresses are returned.  */
 /**/
@@ -32,8 +38,7 @@
 /**/
 #include <signal.h>
 
-#define	EXCEPTIONPROTO							\
-	int sig, int code, struct sigcontext *ctx
+#define	EXCEPTIONPROTO SIGNAL_ARGS(sig, ctx)
 
 /* Get the first exception frame from a signal handler */
 #define	EXCEPTIONFRAME(f, c)						\
@@ -42,4 +47,10 @@
 		(f).retpc = (uintp)(c)->sc_pc;				\
 	} while (0)
 
+/*
+ * Flush the cache on netbsd.
+ */
+#undef	FLUSH_DCACHE
+#define FLUSH_DCACHE(_start,_end) m68k_sync_icache((_start), (uintp)(_end) - (uintp)(_start))
+
 #endif
