@@ -18,6 +18,8 @@
 #ifndef __mips_common_h
 #define __mips_common_h
 
+#include "generic/gentypes.h"
+
 /* The R5900 is the Mips Core in the PS2 (Playstation 2)
 * It has most of the Mips III instructions set, some of the Mips IV 
 * instructions, it lacks the Mips II "ll" and "sc" instructions and 
@@ -35,7 +37,7 @@
 /* Atomicity is temporarily disabled because the compare_and_swap function 
  * do not return the right value. Use the generic one instead.
  */
-#if defined(HAVE_MIPSII_INSTRUCTIONS) && 0
+#if defined(HAVE_MIPSII_INSTRUCTIONS)
 #include "atomic.h"
 #else
 
