@@ -90,29 +90,6 @@ do { \
 #define FIXUP_TRAMPOLINE_DECL Method *_meth, void *_where
 #define FIXUP_TRAMPOLINE_INIT meth = _meth; where = _where;
 
-/*
- * Call a jitted java exception handler.
- */
-#if defined(__APPLE__)
-#define CALL_KAFFE_EXCEPTION(frame, handler, object) \
-	asm volatile ("mr r1, %0\n" \
-		      "\tlwz r10, 0(r1)\n" \
-		      "\tlmw r14, -72(r10)\n" \
-		      "\tmr r3, %1\n" \
-		      "\tmtctr %2\n" \
-		      "\tbctr\n" \
-		      : : "r" (frame), "r" (object), "r" (handler) : "r3")
-#else
-#define CALL_KAFFE_EXCEPTION(frame, handler, object) \
-	asm volatile ("mr 1, %0\n" \
-		      "\tlwz 10, 0(1)\n" \
-		      "\tlmw 14, -72(10)\n" \
-		      "\tmr 3, %1\n" \
-		      "\tmtctr %2\n" \
-		      "\tbctr\n" \
-		      : : "r" (frame), "r" (object), "r" (handler) : "r3")
-#endif
-
 /*
  * Call a JanosVM catastrophe handler.
  * XXX implement me.
