@@ -13,7 +13,7 @@ typedef struct KaffeVM_Arguments {
         jint            verifyMode;
         const char*     classpath;
         const char*     bootClasspath;
-        jint            (*vfprintf)(FILE*, const char*, va_list);
+        jint            (*jni_vfprintf)(FILE*, const char*, va_list);
         void            (*exit)(jint);
         void            (*abort)(void);
         jint            enableClassGC;
@@ -25,6 +25,8 @@ typedef struct KaffeVM_Arguments {
         jint            allocHeapSize;
         const char*     classhome;
         const char*     libraryhome;
+        const char*     profilerLibname;
+        const char*     profilerArguments;
 } KaffeVM_Arguments;
 
 extern KaffeVM_Arguments Kaffe_JavaVMArgs;
