@@ -54,6 +54,8 @@ Utf8Const* ConstantValue_name;
 Utf8Const* Exceptions_name;
 Utf8Const* SourceFile_name;
 Utf8Const* InnerClasses_name;
+Utf8Const* Signature_name;
+Utf8Const* Synthetic_name;
 
 FILE* include;
 FILE* jni_include;
@@ -99,17 +101,19 @@ main(int argc, char* argv[])
 	}
 
 	utf8ConstInit();
-	init_name = utf8ConstNew("<clinit>", -1);
-	final_name = utf8ConstNew("finalize", -1);
-	void_signature = utf8ConstNew("()V", -1);
-	constructor_name = utf8ConstNew("<init>", -1);
-	Code_name = utf8ConstNew("Code", -1);
-	LineNumberTable_name = utf8ConstNew("LineNumberTable", -1);
-	LocalVariableTable_name = utf8ConstNew("LocalVariableTable", -1);
-	ConstantValue_name = utf8ConstNew("ConstantValue", -1);
-	Exceptions_name = utf8ConstNew("Exceptions", -1);
-	SourceFile_name = utf8ConstNew("SourceFile", -1);
-	InnerClasses_name = utf8ConstNew("InnerClasses", -1);
+	init_name = utf8ConstFromString("<clinit>");
+	final_name = utf8ConstFromString("finalize");
+	void_signature = utf8ConstFromString("()V");
+	constructor_name = utf8ConstFromString("<init>");
+	Code_name = utf8ConstFromString("Code");
+	LineNumberTable_name = utf8ConstFromString("LineNumberTable");
+	LocalVariableTable_name = utf8ConstFromString("LocalVariableTable");
+	ConstantValue_name = utf8ConstFromString("ConstantValue");
+	Exceptions_name = utf8ConstFromString("Exceptions");
+	SourceFile_name = utf8ConstFromString("SourceFile");
+	InnerClasses_name = utf8ConstFromString("InnerClasses");
+	Synthetic_name = utf8ConstFromString("Synthetic");
+	Signature_name = utf8ConstFromString("Signature");
 
 	/* Process each class */
 	for (nm = argv[farg]; nm != 0; nm = argv[++farg]) {
@@ -245,7 +249,7 @@ options(int argc, char** argv)
 	bootclasspath = malloc(1);
 	if (bootclasspath == NULL) 
 	  {
-	    fprintf(stderr,  _("Error: out of memory.\n"));
+	    fprintf(stderr, "%s", _("Error: out of memory.\n"));
 	    exit(EXIT_FAILURE);
 	  }
 	*bootclasspath = '\0';
@@ -253,7 +257,7 @@ options(int argc, char** argv)
 	classpath = malloc(1);
 	if (classpath == NULL) 
 	  {
-	    fprintf(stderr,  _("Error: out of memory.\n"));
+	    fprintf(stderr, "%s",  _("Error: out of memory.\n"));
 	    exit(EXIT_FAILURE);
 	  }
 	*classpath = '\0';
@@ -300,7 +304,7 @@ options(int argc, char** argv)
 		  newcp = malloc(cplen);
 		  if (newcp == NULL)
 		    {
-		      fprintf(stderr,  _("Error: out of memory.\n"));
+		      fprintf(stderr, "%s", _("Error: out of memory.\n"));
 		      exit(EXIT_FAILURE);
 		    }
 		  
@@ -325,7 +329,7 @@ options(int argc, char** argv)
 		  newcp = malloc(cplen);
 		  if (newcp == NULL)
 		    {
-		      fprintf(stderr,  _("Error: out of memory.\n"));
+		      fprintf(stderr, "%s", _("Error: out of memory.\n"));
 		      exit(EXIT_FAILURE);
 		    }
 		  
