@@ -59,6 +59,9 @@ Utf8Const* ConstantValue_name;
 Utf8Const* Exceptions_name;
 Utf8Const* SourceFile_name;
 Utf8Const* InnerClasses_name;
+Utf8Const* Signature_name;
+Utf8Const* EnclosingMethod_name;
+Utf8Const* Synthetic_name;
 
 static struct _dispatchTable *ClassClass_vtable;
 static Hjava_lang_Class*	ClassClass;
@@ -66,7 +69,11 @@ Hjava_lang_Class* StringClass;
 Hjava_lang_Class* ObjectClass;
 Hjava_lang_Class* SystemClass;
 Hjava_lang_Class* RuntimeClass;
+/* Note: All interfaces implemented by java.lang.Class must be here. */
 Hjava_lang_Class* SerialClass;
+Hjava_lang_Class* TypeClass;
+Hjava_lang_Class* AnnotatedElementClass;
+Hjava_lang_Class* GenericDeclarationClass;
 Hjava_lang_Class* CloneClass;
 Hjava_lang_Class* PtrClass;
 Hjava_lang_Class* ClassLoaderClass;
@@ -125,6 +132,9 @@ static Hjava_lang_Class** stateCompleteClass[] = {
 #define RUNTIMECLASS "java/lang/Runtime"
 #define SYSTEMCLASS  "java/lang/System"
 #define	SERIALCLASS  "java/io/Serializable"
+#define	TYPECLASS    "java/lang/reflect/Type"
+#define	ANNOTATEDELEMENTCLASS "java/lang/reflect/AnnotatedElement"
+#define	GENERICDECLARATIONCLASS "java/lang/reflect/GenericDeclaration"
 #define	CLONECLASS   "java/lang/Cloneable"
 #define	LOADERCLASS  "java/lang/ClassLoader"
 #define PTRCLASS     "org/kaffe/util/Ptr"
@@ -244,22 +254,26 @@ initialiseKaffe(void)
 	/* Create the initialise and finalize names and signatures. */
 	DBG(INIT, dprintf("create names and signatures\n"); );
 
-	init_name = utf8ConstNew("<clinit>", -1);
-	final_name = utf8ConstNew("finalize", -1);
-	void_signature = utf8ConstNew("()V", -1);
-	constructor_name = utf8ConstNew("<init>", -1);
-	Code_name = utf8ConstNew("Code", -1);
-	LineNumberTable_name = utf8ConstNew("LineNumberTable", -1);
-	LocalVariableTable_name = utf8ConstNew("LocalVariableTable", -1);
-	ConstantValue_name = utf8ConstNew("ConstantValue", -1);
-	Exceptions_name = utf8ConstNew("Exceptions", -1);
-	SourceFile_name = utf8ConstNew("SourceFile", -1);
-	InnerClasses_name = utf8ConstNew("InnerClasses", -1);
+	init_name = utf8ConstFromString("<clinit>");
+	final_name = utf8ConstFromString("finalize");
+	void_signature = utf8ConstFromString("()V");
+	constructor_name = utf8ConstFromString("<init>");
+	Code_name = utf8ConstFromString("Code");
+	LineNumberTable_name = utf8ConstFromString("LineNumberTable");
+	LocalVariableTable_name = utf8ConstFromString("LocalVariableTable");
+	ConstantValue_name = utf8ConstFromString("ConstantValue");
+	Exceptions_name = utf8ConstFromString("Exceptions");
+	SourceFile_name = utf8ConstFromString("SourceFile");
+	InnerClasses_name = utf8ConstFromString("InnerClasses");
+	Signature_name = utf8ConstFromString("Signature");
+	Synthetic_name = utf8ConstFromString("Synthetic");
+	EnclosingMethod_name = utf8ConstFromString("EnclosingMethod");
 
 	if (!(init_name && final_name && void_signature &&
 	      constructor_name && Code_name && LineNumberTable_name &&
 	      LocalVariableTable_name && ConstantValue_name &&
-	      Exceptions_name && SourceFile_name && InnerClasses_name)) {
+	      Exceptions_name && SourceFile_name && InnerClasses_name && 
+	      Signature_name && Synthetic_name && EnclosingMethod_name)) {
 		DBG(INIT, dprintf("not enough memory to run kaffe\n"); );
 		KAFFEVM_ABORT();
 	}
@@ -285,12 +299,12 @@ initialiseKaffe(void)
 	/* Setup exceptions */
 	initExceptions();
 
-	/* Init stuff for the java security model */
-	initialiseSecurity();
-
 	/* Init thread support */
 	initThreads();
 
+	/* Init stuff for the java security model */
+	initialiseSecurity();
+
 	/* Now enable collector */
 	KGC_enable(main_collector);
 }
@@ -333,12 +347,18 @@ initBaseClasses(void)
 
 	loadStaticClass(&ObjectClass, OBJECTCLASS);
 	loadStaticClass(&SerialClass, SERIALCLASS);
+	loadStaticClass(&TypeClass,   TYPECLASS);
+	loadStaticClass(&AnnotatedElementClass, ANNOTATEDELEMENTCLASS);
+	loadStaticClass(&GenericDeclarationClass, GENERICDECLARATIONCLASS);
 	loadStaticClass(&CloneClass,  CLONECLASS);
 	loadStaticClass(&ClassClass,  CLASSCLASS);
 
 	ClassClass_vtable = ClassClass->vtable;
 	ObjectClass->head.vtable = ClassClass_vtable;
 	SerialClass->head.vtable = ClassClass_vtable;
+	TypeClass->head.vtable   = ClassClass_vtable;
+	AnnotatedElementClass->head.vtable = ClassClass_vtable;
+	GenericDeclarationClass->head.vtable = ClassClass_vtable;
 	CloneClass->head.vtable  = ClassClass_vtable;
 	ClassClass->head.vtable  = ClassClass_vtable;
 
