@@ -18,24 +18,23 @@
 #define	STRINGCLASS	"java/lang/String"
 #define	OBJECTCLASS	"java/lang/Object"
 #define	CLASSCLASS	"java/lang/Class"
-#define	APPCLASSLOADERCLASS "kaffe/lang/AppClassLoader"
 
 #define	OBJECTCLASSSIG	"Ljava/lang/Object;"
 
-#define	PTRCLASS	"kaffe/util/Ptr"
-#define	PTRCLASSSIG	"Lkaffe/util/Ptr;"
+#define	PTRCLASS	"org/kaffe/util/Ptr"
+#define	PTRCLASSSIG	"Lorg/kaffe/util/Ptr;"
 
 void initBaseClasses(void);
 void initialiseKaffe(void);
 
 extern struct Hjava_lang_Class*	ObjectClass;
 extern struct Hjava_lang_Class*	StringClass;
+extern struct Hjava_lang_Class*	RuntimeClass;
 extern struct Hjava_lang_Class*	SystemClass;
 extern struct Hjava_lang_Class*	SerialClass;
 extern struct Hjava_lang_Class*	CloneClass;
 extern struct Hjava_lang_Class* PtrClass;
 extern struct Hjava_lang_Class* ClassLoaderClass;
-extern struct Hjava_lang_Class* kaffeLangAppClassLoaderClass;
 
 extern struct Hjava_lang_Class* javaLangRefReference;
 extern struct Hjava_lang_Class* javaLangRefSoftReference;
@@ -52,6 +51,14 @@ extern struct Hjava_lang_Class*	javaLangLongClass;
 extern struct Hjava_lang_Class*	javaLangFloatClass;
 extern struct Hjava_lang_Class*	javaLangDoubleClass;
 
+extern struct Hjava_lang_Class *javaNioDirectByteBufferImplClass;
+extern struct Hjava_lang_Class *javaNioDirectByteBufferImplReadWriteClass;
+extern struct Hjava_lang_Class *javaNioBufferClass;
+extern struct Hjava_lang_Class *gnuClasspathPointerClass;
+
+extern Field *gnuClasspathPointerAddress;
+extern Field *directByteBufferImplAddress;
+
 extern struct Hjava_lang_Class* javaLangThrowable;
 extern struct Hjava_lang_Class* javaLangVMThrowable;
 extern struct Hjava_lang_Class* javaLangStackTraceElement;
@@ -63,9 +70,7 @@ extern struct Hjava_lang_Class* javaLangClassNotFoundException;
 extern struct Hjava_lang_Class* javaLangNoClassDefFoundError;
 extern struct Hjava_lang_Class* javaLangStackOverflowError;
 
-extern struct Hjava_lang_ClassLoader *appClassLoader;
-
-extern Hjava_lang_Class* getClassClass(void);
+extern struct _dispatchTable* getClassVtable(void);
 extern Hjava_lang_Class* getStringClass(void);
 
 #endif
