@@ -7,7 +7,7 @@
 #ifndef _GC_Kaffe_H
 #define _GC_Kaffe_H
 
-#include "boehm/include/gc_mark.h"
+#include <gc/gc_mark.h>
 
 void * GC_kaffe_malloc(size_t lb);
 void GC_kaffe_init(GC_mark_proc proc);
