@@ -16,6 +16,7 @@
 #include "config-hacks.h"
 #include "defs.h"
 #include "gtypes.h"
+#include "machine.h"
 #include "slots.h"
 #include "access.h"
 #include "object.h"
@@ -50,7 +51,7 @@
 #include "methodcalls.h"
 
 #if 0
-#define	METHOD_TRUE_NCODE(METH)			(METH)->c.ncode.ncode_start
+#define	METHOD_TRUE_NCODE(METH)			getMethodCodeStart(METH)
 #define	METHOD_PRE_COMPILED(METH)		((int16)(METH)->localsz < 0)
 #define	SET_METHOD_PRE_COMPILED(METH, VAL)	((METH)->localsz = -(VAL))
 #endif
@@ -86,6 +87,8 @@ initialiseSecurity (void)
 	struct Hjava_lang_Object *permissions;
 	struct Hjava_lang_Object *allPermission;
 
+        DBG(INIT, dprintf("initialiseSecurity()\n"); );
+
 	codeSource = execute_java_constructor ("java/security/CodeSource",
 					       NULL,
 					       NULL,
@@ -113,6 +116,7 @@ initialiseSecurity (void)
 							    "(Ljava/security/CodeSource;Ljava/security/PermissionCollection;)V",
 							    codeSource,
 							    permissions);
+        DBG(INIT, dprintf("initialiseSecurity() done\n"); );
 }
 
 #if !defined(ALIGNMENT_OF_SIZE)
@@ -244,29 +248,29 @@ retry:
 				       class->superclass, 
 				       (void **)&(class->superclass));
 			if( !(class->accflags & ACC_INTERFACE) &&
-			    (class->superclass->accflags & ACC_INTERFACE)) {
+			    (getSuperclass(class)->accflags & ACC_INTERFACE)) {
 				postExceptionMessage(
 					einfo,
 					JAVA_LANG(
 						IncompatibleClassChangeError),
 					"Super class, %s, is an interface.",
-					class->superclass->name->data);
+					getSuperclass(class)->name->data);
 				success = false;
 				goto done;
 			}
 			/* that's pretty much obsolete. */
-			assert(class->superclass->state >= CSTATE_DOING_LINK);
+			assert(getSuperclass(class)->state >= CSTATE_DOING_LINK);
 			classMappingLoaded(ce, class);
 			/* Copy initial field size and gc layout.
 			 * Later, as this class's fields are resolved, they
 			 * are added to the superclass's layout.
 			 */
-			CLASS_FSIZE(class) = CLASS_FSIZE(class->superclass);
-			class->gc_layout = class->superclass->gc_layout;
+			CLASS_FSIZE(class) = CLASS_FSIZE(getSuperclass(class));
+			class->gc_layout = getSuperclass(class)->gc_layout;
 		}
 		if( class->superclass )
 		{
-			assert(class->superclass->state >= CSTATE_DOING_LINK);
+			assert(getSuperclass(class)->state >= CSTATE_DOING_LINK);
 		}
 		
 	}
@@ -526,7 +530,7 @@ retry:
 					       einfo);
 			lockClass(class);
 			if (success == false) {
-				if (class->superclass->state == CSTATE_FAILED)
+				if (getSuperclass(class)->state == CSTATE_FAILED)
 					SET_CLASS_STATE(CSTATE_FAILED);
 				goto done;
 			}
@@ -630,7 +634,7 @@ DBG(STATICINIT, dprintf("Initialising %s static %d\n", class->name->data,
 #if defined(TRANSLATOR) && (defined (MD_UNREGISTER_JIT_EXCEPTION_INFO) || defined (JIT3))
 #if defined(MD_UNREGISTER_JIT_EXCEPTION_INFO)
 		if (exc == 0) {
-			MD_UNREGISTER_JIT_EXCEPTION_INFO (meth->c.ncode.ncode_start,
+			MD_UNREGISTER_JIT_EXCEPTION_INFO (getMethodCodeStart(meth),
 							  METHOD_NATIVECODE(meth),
 							  meth->c.ncode.ncode_end);
 		}
@@ -645,7 +649,7 @@ DBG(STATICINIT, dprintf("Initialising %s static %d\n", class->name->data,
 #endif
 		    1) {
 			_SET_METHOD_NATIVECODE(meth, NULL);
-			meth->c.ncode.ncode_start = NULL;
+			setMethodCodeStart(meth, NULL);
 			meth->c.ncode.ncode_end = NULL;
 		}
 	}
@@ -698,14 +702,14 @@ expandMethods(Hjava_lang_Class *cl, Method *imeth, errorInfo *einfo)
 	}
 	else
 	*/
-	if( (new_methods = gc_realloc(CLASS_METHODS(cl),
+	if( (new_methods = gc_realloc(Kaffe_get_class_methods(cl),
 				      sizeof(Method) *
 				      (CLASS_NMETHODS(cl) + 1), KGC_ALLOC_METHOD)) )
 	{
 		int i;
 		
 		i = CLASS_NMETHODS(cl);
-		CLASS_METHODS(cl) = new_methods;
+		Kaffe_set_class_methods(cl, new_methods);
 		utf8ConstAddRef(imeth->name);
 		new_methods[i] = *imeth;
 
@@ -764,7 +768,7 @@ expandInterfaces(Hjava_lang_Class *root_class,
 			int foundit = 0;
 			Method *imeth;
 			
-			imeth = &CLASS_METHODS(iface)[j];
+			imeth = &(Kaffe_get_class_methods(iface)[j]);
 			/* Igore statics */
 			if( imeth->accflags & ACC_STATIC )
 				continue;
@@ -777,7 +781,7 @@ expandInterfaces(Hjava_lang_Class *root_class,
 				{
 					Method *cmeth;
 					
-					cmeth = &CLASS_METHODS(cl)[k];
+					cmeth = &(Kaffe_get_class_methods(cl)[k]);
 					if( (cmeth->name == imeth->name) &&
 					    (cmeth->parsed_sig->signature ==
 					     imeth->parsed_sig->signature) )
@@ -820,7 +824,7 @@ resolveInterfaces(Hjava_lang_Class *class, errorInfo *einfo)
 			success = false;
 			goto done;
 		}
-		j += class->superclass->total_interface_len;
+		j += getSuperclass(class)->total_interface_len;
 	}
 	for (i = 0; i < class->interface_len; i++) {
 		uintp iface = (uintp)class->interfaces[i];
@@ -981,7 +985,7 @@ internalSetupClass(Hjava_lang_Class* cl, Utf8Const* name, int flags,
 		return 0;
 	}
 	cl->packageLength = findPackageLength(name->data);
-	CLASS_METHODS(cl) = NULL;
+	Kaffe_set_class_methods(cl, NULL);
 	CLASS_NMETHODS(cl) = 0;
 	assert(cl->superclass == 0);
 	cl->superclass = (Hjava_lang_Class*)(uintp)su;
@@ -1019,7 +1023,8 @@ setupClass(Hjava_lang_Class* cl, constIndex c, constIndex s,
 	pool = CLASS_CONSTANTS(cl);
 
 	/* Find the name of the class */
-	if (pool->tags[c] != CONSTANT_Class) {
+	if (c >= CLASS_CONST_SIZE (cl)
+	    || pool->tags[c] != CONSTANT_Class) {
                 postExceptionMessage(einfo, JAVA_LANG(ClassFormatError),
 				     "this class constant pool index is bogus");
 		return (NULL);
@@ -1072,6 +1077,7 @@ addSourceFile(Hjava_lang_Class* c, int idx, errorInfo *einfo)
 	return (success);
 }
 
+
 /*
  * Read in InnerClasses declares for a class
  */
@@ -1104,13 +1110,16 @@ addInnerClasses(Hjava_lang_Class* c, size_t len UNUSED, classFile* fp,
 	c->inner_classes = ic;
 
 	for (i = 0; i < nr; i++, ic++) {
-		u2 dummy;
+		u2 inner_name_index;
 
 		readu2(&ic->inner_class, fp);
 		readu2(&ic->outer_class, fp);
-		readu2(&dummy, fp);
+		readu2(&inner_name_index, fp);
 		readu2(&ic->inner_class_accflags, fp);
 
+		if (inner_name_index != 0)
+		  c->kFlags |= KFLAG_ANONYMOUS;
+
 		if (c->this_index && ic->inner_class == c->this_index) {
 		    c->this_inner_index = i;
 		}
@@ -1123,18 +1132,20 @@ startMethods(Hjava_lang_Class* this, u2 methct, errorInfo *einfo)
 {
 	if (methct == 0)
 	{
-		this->methods = NULL;
+		Kaffe_set_class_methods(this, NULL);
 	}
 	else
 	{
-		this->methods = gc_malloc(sizeof(Method)*(methct), KGC_ALLOC_METHOD);
-		if (this->methods == NULL)
+		Kaffe_set_class_methods(this, 
+					gc_malloc(sizeof(Method)*(methct), 
+						  KGC_ALLOC_METHOD));
+		if (Kaffe_get_class_methods(this) == NULL)
 		{
 			postOutOfMemory(einfo);
 			return false;
 		}
 	}
-	KGC_WRITE(this, this->methods);
+	KGC_WRITE(this, Kaffe_get_class_methods(this));
 
 	CLASS_NMETHODS(this) = 0; /* updated in addMethod */
 	return true;
@@ -1160,7 +1171,8 @@ addMethod(Hjava_lang_Class* c,
 	pool = CLASS_CONSTANTS (c);
 
 	nc = name_index;
-	if (pool->tags[nc] != CONSTANT_Utf8) {
+	if (nc >= CLASS_CONST_SIZE (c)
+	    || pool->tags[nc] != CONSTANT_Utf8) {
 		/* XXX fill in einfo! */
 DBG(RESERROR,	dprintf("addMethod: no method name.\n");		);
 		postExceptionMessage(einfo,
@@ -1169,7 +1181,8 @@ DBG(RESERROR,	dprintf("addMethod: no method name.\n");		);
 		return (NULL);
 	}
 	sc = signature_index;
-	if (pool->tags[sc] != CONSTANT_Utf8) {
+	if (sc >= CLASS_CONST_SIZE (c)
+	    || pool->tags[sc] != CONSTANT_Utf8) {
 		/* XXX fill in einfo! */
 DBG(RESERROR,	dprintf("addMethod: no signature name.\n");	);
 		postExceptionMessage(einfo,
@@ -1183,7 +1196,7 @@ DBG(RESERROR,	dprintf("addMethod: no signature name.\n");	);
 
 #ifdef KAFFE_VMDEBUG
 	/* Search down class for method name - don't allow duplicates */
-	for (ni = CLASS_NMETHODS(c), mt = CLASS_METHODS(c); --ni >= 0; ) {
+	for (ni = CLASS_NMETHODS(c), mt = Kaffe_get_class_methods(c); --ni >= 0; ) {
 		assert(! utf8ConstEqual (name, mt->name)
 		       || ! utf8ConstEqual (signature, METHOD_SIG(mt)));
 	}
@@ -1193,15 +1206,14 @@ DBG(CLASSFILE,
 	dprintf("Adding method %s:%s%s (%x)\n", c->name->data, name->data, signature->data, access_flags);
     );
 
-	mt = &CLASS_METHODS(c)[CLASS_NMETHODS(c)];
+	mt = &(Kaffe_get_class_methods(c)[CLASS_NMETHODS(c)]);
 	utf8ConstAssign(mt->name, name);
 	METHOD_PSIG(mt) = parseSignature(signature, einfo);
 	if (METHOD_PSIG(mt) == NULL) {
 		return NULL;
 	}
 	mt->class = c;
-	/* Warning: ACC_CONSTRUCTION match ACC_STRICT */
-	mt->accflags = access_flags & ACC_MASK;
+	mt->accflags = access_flags;
 	mt->c.bcode.code = NULL;
 	mt->stacksz = 0;
 	mt->localsz = 0;
@@ -1210,7 +1222,7 @@ DBG(CLASSFILE,
 
 	/* Mark constructors as such */
 	if (utf8ConstEqual (name, constructor_name)) {
-		mt->accflags |= ACC_CONSTRUCTOR;
+		mt->kFlags |= KFLAG_CONSTRUCTOR;
 	}
 
 	CLASS_NMETHODS(c)++;
@@ -1234,7 +1246,8 @@ addField(Hjava_lang_Class* c,
 	pool = CLASS_CONSTANTS (c);
 
 	nc = name_index;
-	if (pool->tags[nc] != CONSTANT_Utf8) {
+	if (nc >= CLASS_CONST_SIZE (c)
+	    || pool->tags[nc] != CONSTANT_Utf8) {
 DBG(RESERROR,	dprintf("addField: no field name.\n");			);
 		postExceptionMessage(einfo,
 				     JAVA_LANG(ClassFormatError),
@@ -1258,7 +1271,8 @@ DBG(CLASSFILE,
     );
 
 	sc = signature_index;
-	if (pool->tags[sc] != CONSTANT_Utf8) {
+	if (pool->tags[sc] != CONSTANT_Utf8
+	    || sc >=CLASS_CONST_SIZE(c)) {
 DBG(RESERROR,	dprintf("addField: no signature name.\n");		);
 		postExceptionMessage(einfo,
 				     JAVA_LANG(ClassFormatError),
@@ -1605,7 +1619,7 @@ loadStaticClass(Hjava_lang_Class** class, const char* name)
 	classEntry* centry;
 	bool refAdded;
 
-	utf8 = utf8ConstNew(name, -1);
+	utf8 = utf8ConstFromString(name);
 	if (!utf8) goto bad;
 	centry = lookupClassEntry(utf8, NULL, &info);
 	if (!centry) goto bad;
@@ -1657,7 +1671,7 @@ lookupClass(const char* name, Hjava_lang_ClassLoader* loader, errorInfo *einfo)
 	Hjava_lang_Class* class;
 	Utf8Const *utf8;
 
-	utf8 = utf8ConstNew(name, -1);
+	utf8 = utf8ConstFromString(name);
 	if (!utf8) {
 		postOutOfMemory(einfo);
 		return NULL;
@@ -2015,7 +2029,7 @@ getInheritedMethodIndex(Hjava_lang_Class *super, Method *meth)
 	 */
 	for (; super != NULL;  super = super->superclass) {
 		int j = CLASS_NMETHODS(super);
-		Method* mt = CLASS_METHODS(super);
+		Method* mt = Kaffe_get_class_methods(super);
 		for (; --j >= 0;  ++mt) {
 			/* skip methods that are private or static */
 			if ((mt->accflags & (ACC_PRIVATE|ACC_STATIC)) != 0)
@@ -2042,7 +2056,7 @@ getInheritedMethodIndex(Hjava_lang_Class *super, Method *meth)
  * @param method a method
  * @return pointer to start of code
  */
-struct _jitCodeHeader*
+jitCodeHeader*
 getMethodCodeStart(Method * method) 
 {
   return method->c.ncode.ncode_start;
@@ -2055,7 +2069,7 @@ getMethodCodeStart(Method * method)
  * @param start pointer to start of code
  */
 void
-setMethodCodeStart(Method * method, struct _jitCodeHeader* start)
+setMethodCodeStart(Method * method, jitCodeHeader* start)
 {
   method->c.ncode.ncode_start = start;
 }
@@ -2070,13 +2084,13 @@ buildDispatchTable(Hjava_lang_Class* class, errorInfo *einfo)
 	Hjava_lang_Class *cc;
 
 	if (class->superclass != NULL) {
-		class->msize = class->superclass->msize;
+		class->msize = getSuperclass(class)->msize;
 	}
 	else {
 		class->msize = 0;
 	}
 
-	meth = CLASS_METHODS(class);
+	meth = Kaffe_get_class_methods(class);
 	i = CLASS_NMETHODS(class);
 	for (; --i >= 0; meth++) {
 		Hjava_lang_Class* super = class->superclass;
@@ -2121,7 +2135,7 @@ buildDispatchTable(Hjava_lang_Class* class, errorInfo *einfo)
 	mtab = class->vtable->method;
 
 	/* now build a trampoline for each and every method */
-	meth = CLASS_METHODS(class);
+	meth = Kaffe_get_class_methods(class);
 	i = CLASS_NMETHODS(class);
 
 	for (; --i >= 0; meth++) {
@@ -2141,7 +2155,7 @@ buildDispatchTable(Hjava_lang_Class* class, errorInfo *einfo)
 	 * methods so they can be patched up independently
 	 */
 	for (cc = class->superclass; cc != 0; cc = cc->superclass) {
-		meth = CLASS_METHODS(cc);
+		meth = Kaffe_get_class_methods(cc);
 		i = CLASS_NMETHODS(cc);
 		for (; --i >= 0; meth++) {
 			void **where;
@@ -2221,7 +2235,7 @@ buildInterfaceDispatchTable(Hjava_lang_Class* class, errorInfo *einfo)
 	j = 1;
 	for (i = 0; i < class->total_interface_len; i++) {
 		int inm = CLASS_NMETHODS(class->interfaces[i]);
-		Method *imeth = CLASS_METHODS(class->interfaces[i]);
+		Method *imeth = Kaffe_get_class_methods(class->interfaces[i]);
 
 		/* store interface as first word for type inclusion test */
 		class->itable2dtable[j++] = class->interfaces[i];
@@ -2241,7 +2255,7 @@ buildInterfaceDispatchTable(Hjava_lang_Class* class, errorInfo *einfo)
 			 */
 			for (ncl = class; ncl != NULL;  ncl = ncl->superclass) {
 				int k = CLASS_NMETHODS(ncl);
-				cmeth = CLASS_METHODS(ncl);
+				cmeth = Kaffe_get_class_methods(ncl);
 				for (; --k >= 0;  ++cmeth) {
 					if (utf8ConstEqual (cmeth->name,
 							    imeth->name) &&
@@ -2470,7 +2484,7 @@ prepareInterface(Hjava_lang_Class* class, errorInfo *einfo)
 	Method* meth;
 	int i;
 
-	meth = CLASS_METHODS(class);
+	meth = Kaffe_get_class_methods(class);
 	class->msize = 0;
 
 	/* enumerate indices and store them in meth->idx */
@@ -2886,7 +2900,7 @@ lookupArray(Hjava_lang_Class* c, errorInfo *einfo)
 			return (NULL);
 		}
 
-		arr_class = CLASS_ARRAY_CACHE(c);
+		arr_class = Kaffe_get_primitive_array_cache(c);
 		if (arr_class) {
 			return (arr_class);
 		}
@@ -2903,7 +2917,7 @@ lookupArray(Hjava_lang_Class* c, errorInfo *einfo)
 			sprintf (sig, "[L%s;", cname);
 		}
 	}
-	arr_name = utf8ConstNew(sig, -1);	/* release before returning */
+	arr_name = utf8ConstFromString(sig);	/* release before returning */
 	KFREE(sig);
 
 	if (!arr_name) {
@@ -2964,7 +2978,7 @@ lookupArray(Hjava_lang_Class* c, errorInfo *einfo)
 		centry->data.cl = c = NULL;
 		goto bail;
 	}
-	CLASS_ELEMENT_TYPE(arr_class) = c;
+	Kaffe_set_array_element_type(arr_class, c);
 
 	/* Add the interfaces that arrays implement.  Note that addInterface
 	 * will hold on to arr_interfaces, so it must be a static variable.
@@ -2989,7 +3003,7 @@ bail:
 
 	found:;
 	if (c && CLASS_IS_PRIMITIVE(c)) {
-		CLASS_ARRAY_CACHE(c) = centry->data.cl;
+		Kaffe_set_primitive_array_cache(c, centry->data.cl);
 	}
 	
 #if defined(KAFFE_XDEBUGGING)
