@@ -12,7 +12,11 @@
 #ifndef __classmethod_h
 #define __classmethod_h
 
+#include <assert.h>
+
+#include "config.h"
 #include "gtypes.h"
+#include "machine.h"
 #include "access.h"
 #include "object.h"
 #include "md.h"	 /* XXX: need this here so KAFFE_PROFILER is accurately def'd */
@@ -45,7 +49,6 @@ typedef enum {
 struct _classEntry;
 struct _innerClass;
 struct Hjava_lang_String;
-struct _jitCodeHeader;
 
 #include <java_lang_ClassLoader.h>
 #include "reference.h"
@@ -99,21 +102,29 @@ struct Hjava_lang_Class {
 	struct _classEntry*	centry;
 
 	Utf8Const*		name;
+        Utf8Const*              extSignature;
+        constIndex              enclosingClassIndex;
+        constIndex              enclosingMethodIndex;
 	unsigned int		packageLength;
 	char*			sourcefile;	/* source file name if known */
 	accessFlags		accflags;
+        kaffeClassFlags         kFlags;        
 
 	/* If non-NULL, a pointer to the superclass.
 	 * However, if state < CSTATE_DOING_PREPARE, then
 	 * (int) superclass is a constant pool index. */
-	struct Hjava_lang_Class* superclass;
+	void*                   superclass;
 
 	struct _constants	constants;
 
 	/* For regular classes, an array of the methods defined in this class.
 	   For array types, used for CLASS_ELEMENT_TYPE.
 	   For primitive types, used by CLASS_ARRAY_CACHE. */
-	Method*			methods;
+        union {
+	  Method*		       methods;
+	  struct Hjava_lang_Class*     element_type;
+	  struct Hjava_lang_Class*     array_cache;
+	} cache;
 	short			method_count;
 
 	/* Number of methods in the dtable. */
@@ -144,7 +155,7 @@ struct Hjava_lang_Class {
 	unsigned short		total_interface_len;
 	/* pointers to all itable2dtable entries of classes implementing this interface */
 	void***			implementors;	/* interfaces only */
-        int			impl_index;    /* index of the class in the implementors array. */
+        unsigned int		impl_index;    /* index of the class in the implementors array. */
 
 	Hjava_lang_ClassLoader*	loader;
 
@@ -197,8 +208,8 @@ typedef struct Hjava_lang_Class Hjava_lang_Class;
 #define signalOnClass(C) signalCond(((C)))
 #define broadcastOnClass(C) broadcastCond(((C)))
 
-#define METHOD_TRANSLATED(M)		((M)->accflags & ACC_TRANSLATED)
-#define METHOD_JITTED(M)		((M)->accflags & ACC_JITTED)
+#define METHOD_TRANSLATED(M)		((M)->kFlags & KFLAG_TRANSLATED)
+#define METHOD_JITTED(M)		((M)->kFlags & KFLAG_JITTED)
 
 #define	METHOD_NATIVECODE(M)		(((M)->idx == -1) ? \
 		((M)->ncode) : \
@@ -217,7 +228,7 @@ typedef struct Hjava_lang_Class Hjava_lang_Class;
  * @param method a method
  * @return pointer to start of code
  */
-struct _jitCodeHeader* getMethodCodeStart(Method * method);
+jitCodeHeader* getMethodCodeStart(Method * method);
 
 /**
  * Set start of native code of a method
@@ -225,7 +236,7 @@ struct _jitCodeHeader* getMethodCodeStart(Method * method);
  * @param method a method
  * @param start pointer to start of code
  */
-void setMethodCodeStart(Method * method, struct _jitCodeHeader* start);
+void setMethodCodeStart(Method * method, jitCodeHeader* start);
 
 #define	_SET_METHOD_NATIVECODE(M, C)	do {\
 	if ((M)->idx == -1) {\
@@ -235,9 +246,9 @@ void setMethodCodeStart(Method * method, struct _jitCodeHeader* start);
 	} \
 } while (0)
 #define	SET_METHOD_NATIVECODE(M, C)	_SET_METHOD_NATIVECODE(M, C); \
-					(M)->accflags |= ACC_TRANSLATED
+					(M)->kFlags |= KFLAG_TRANSLATED
 #define	SET_METHOD_JITCODE(M, C)	_SET_METHOD_NATIVECODE(M, C); \
-					(M)->accflags |= ACC_TRANSLATED|ACC_JITTED
+					(M)->kFlags |= KFLAG_TRANSLATED|KFLAG_JITTED;
 
 /*
  * Stats for the nameMapping object.
@@ -306,7 +317,9 @@ typedef struct _parsed_signature {
 typedef struct _jmethodID {
 	Utf8Const*		name;
 	parsed_signature_t*	parsed_sig;
+        Utf8Const*              extSignature;
 	accessFlags		accflags;
+        kaffeClassFlags         kFlags;
 	long			idx;	/* Index into class->vtable */
 	u2			stacksz;
 	u2			localsz;
@@ -314,12 +327,12 @@ typedef struct _jmethodID {
 	nativecode*		ncode;
 	union {
 	  struct {
-		struct _jitCodeHeader*	ncode_start;
+		jitCodeHeader*	ncode_start;
 		nativecode*	ncode_end;
 	  } ncode;
 	  struct {
 		unsigned char*	code;
-		int		codelen;
+		unsigned int	codelen;
 	  } bcode;
 	} c;
 	Hjava_lang_Class*	class;
@@ -376,6 +389,7 @@ typedef struct _jfieldID {
 	Hjava_lang_Class*	clazz;
 	Utf8Const*		name;
 	Utf8Const*		signature;
+        Utf8Const*              extSignature;
 	Hjava_lang_Class*	type;
 	accessFlags		accflags;
 	u2			bsize;		/* in bytes */
@@ -412,7 +426,6 @@ struct _method_info;
 struct _field_info;
 struct classFile;
 
-#define CLASS_METHODS(CLASS)  ((CLASS)->methods)
 #define CLASS_NMETHODS(CLASS)  ((CLASS)->method_count)
 
 /* An array containing all the Fields, static fields first. */
@@ -460,11 +473,97 @@ struct classFile;
 #define	CLASS_IS_ABSTRACT(CL) ((CL)->accflags & ACC_ABSTRACT)
 #define	CLASS_IS_FINAL(CL) ((CL)->accflags & ACC_FINAL)
 
-/* For an array type, the types of the elements. */
-#define CLASS_ELEMENT_TYPE(ARRAYCLASS) (*(Hjava_lang_Class**)&(ARRAYCLASS)->methods)
+/**
+ * get array of methods in class.
+ * 
+ * @param clazz the class
+ *
+ * @return methods in the class
+ */
+static inline 
+Method * 
+Kaffe_get_class_methods(struct Hjava_lang_Class * clazz)
+{
+  return clazz->cache.methods;
+}
 
-/* Used by the lookupArray function. */
-#define CLASS_ARRAY_CACHE(PRIMTYPE) (*(Hjava_lang_Class**)&(PRIMTYPE)->methods)
+/**
+ * set array of methods in class.
+ *
+ * @param clazz the class
+ * @param methods the methods to set
+ */
+static inline 
+void 
+Kaffe_set_class_methods(struct Hjava_lang_Class * clazz,
+			Method * methods)
+{
+  clazz->cache.methods = methods;
+}
+
+/**
+ * get type of elements in an array class.
+ * 
+ * @param arrayclazz the class
+ *
+ * @return  type of elements in the array class
+ */
+static inline 
+Hjava_lang_Class * 
+Kaffe_get_array_element_type(struct Hjava_lang_Class * arrayclazz)
+{
+  assert(CLASS_IS_ARRAY(arrayclazz));
+
+  return arrayclazz->cache.element_type;
+}
+
+/**
+ * set type of elements in an array class.
+ *
+ * @param arrayclazz the class
+ * @param element_type the type to set
+ */
+static inline 
+void 
+Kaffe_set_array_element_type(struct Hjava_lang_Class * arrayclazz,
+			     Hjava_lang_Class * element_type)
+{
+  assert(CLASS_IS_ARRAY(arrayclazz));
+
+  arrayclazz->cache.element_type = element_type;
+}
+
+/**
+ * get array cache of a primitive class.
+ * 
+ * @param primitive_clazz the class
+ *
+ * @return  array_cache of the primitive class
+ */
+static inline 
+Hjava_lang_Class * 
+Kaffe_get_primitive_array_cache(struct Hjava_lang_Class * primitive_clazz)
+{
+  assert(CLASS_IS_PRIMITIVE(primitive_clazz));
+
+  return primitive_clazz->cache.array_cache;
+}
+
+/**
+ * set array cache in primitive class.
+ *
+ * @param primitive_clazz the class
+ * @param array_cache the new value
+ */
+static inline 
+void 
+Kaffe_set_primitive_array_cache(struct Hjava_lang_Class * primitive_clazz,
+			     Hjava_lang_Class * array_cache)
+{
+  assert(CLASS_IS_PRIMITIVE(primitive_clazz));
+
+  primitive_clazz->cache.array_cache = array_cache;
+}
 
 #define TYPE_PRIM_SIZE(CL) ((CL)->size_in_bytes)
 #define TYPE_SIZE(CL) \
@@ -475,7 +574,7 @@ struct classFile;
 #define METHOD_IS_PROTECTED(METH)    ((METH)->accflags & ACC_PROTECTED)
 #define METHOD_IS_PRIVATE(METH)      ((METH)->accflags & ACC_PRIVATE)
 
-#define METHOD_IS_CONSTRUCTOR(METH)  ((METH)->accflags & ACC_CONSTRUCTOR)
+#define METHOD_IS_CONSTRUCTOR(METH)  ((METH)->kFlags & KFLAG_CONSTRUCTOR)
 #define METHOD_IS_STATIC(METH)       ((METH)->accflags & ACC_STATIC)
 
 #define METHOD_IS_ABSTRACT(METH)     ((METH)->accflags & ACC_ABSTRACT)
@@ -486,8 +585,8 @@ struct classFile;
 #define METHOD_IS_SYNCHRONISED(METH) ((METH)->accflags & ACC_SYNCHRONISED)
 
 
-#define CLASS_GCJ(C)		((C)->accflags & ACC_GCJ)
-#define SET_CLASS_GCJ(C)	(C)->accflags |= ACC_GCJ
+#define CLASS_GCJ(C)		((C)->kFlags & KFLAG_GCJ)
+#define SET_CLASS_GCJ(C)	(C)->kFlags |= KFLAG_GCJ
 
 /* For manipulating the constant pool in a class */
 #define CLASS_CONSTANTS(CL) (&(CL)->constants)
@@ -691,7 +790,22 @@ extern Utf8Const* ConstantValue_name;	/* "ConstantValue" */
 extern Utf8Const* Exceptions_name;	/* "Exceptions" */
 extern Utf8Const* SourceFile_name;	/* "SourceFile" */
 extern Utf8Const* InnerClasses_name;	/* "InnerClasses" */
+extern Utf8Const* Signature_name;       /* "Signature" */
+extern Utf8Const* Synthetic_name;       /* "Synthetic" */
+extern Utf8Const* EnclosingMethod_name;  /* "EnclosingMethod" */
 
 void initialiseSecurity (void);
 
+/**
+ * get super class for a given class.
+ *
+ * @param class the class
+ *
+ * @return superclass of class
+ */
+static inline struct Hjava_lang_Class* getSuperclass(struct Hjava_lang_Class* class)
+{
+  return class->superclass;
+}
+
 #endif
