@@ -107,8 +107,8 @@ addCode(Method* m, size_t len UNUSED, classFile* fp, errorInfo *einfo)
 				     "Code of a method has length 0",
 				     m->class->name->data,
 				     m->name->data,
-				     m->class->
-				     methods[m - CLASS_METHODS(m->class)].
+				     Kaffe_get_class_methods(m->class)
+				     [m - Kaffe_get_class_methods(m->class)].
 				     parsed_sig->signature->data);
 	}
 	else
@@ -119,8 +119,8 @@ addCode(Method* m, size_t len UNUSED, classFile* fp, errorInfo *einfo)
 				     "Code of a method longer than 65535 bytes",
 				     m->class->name->data,
 				     m->name->data,
-				     m->class->
-				     methods[m - CLASS_METHODS(m->class)].
+				     Kaffe_get_class_methods(m->class)
+				     [m - Kaffe_get_class_methods(m->class)].
 				     parsed_sig->signature->data);
 	}
 	return (retval);
@@ -143,6 +143,13 @@ addLineNumbers(Method* m, size_t len UNUSED, classFile* fp, errorInfo *info)
 
 	lines = gc_malloc(sizeof(lineNumbers)+sizeof(lineNumberEntry) * nr, KGC_ALLOC_LINENRTABLE);
 	if (!lines) {
+	  DBG(READCLASS,
+	      dprintf("%s (%s): failed to allocate %d bytes for %d line number table entries.\n",
+		      CLASS_CNAME(m->class), m->name->data,
+		      sizeof(lineNumbers)+sizeof(lineNumberEntry) * nr,
+		      nr);
+	      );
+
 		postOutOfMemory(info);
 		return false;
 	}
@@ -153,7 +160,12 @@ addLineNumbers(Method* m, size_t len UNUSED, classFile* fp, errorInfo *info)
 		lines->entry[i].start_pc = data;
 		readu2(&data, fp);
 		lines->entry[i].line_nr = data;
-		if (lines->entry[i].start_pc >= m->c.bcode.codelen) {
+		if (lines->entry[i].start_pc > m->c.bcode.codelen) {
+		  DBG(READCLASS,
+		      dprintf("%s (%s): start pc %d of line number entry %d > method length %d.\n",
+			      CLASS_CNAME(m->class), m->name->data,
+			      lines->entry[i].start_pc, i, m->c.bcode.codelen);
+		      );
 			postExceptionMessage(info,
 					     JAVA_LANG(ClassFormatError),
 					     "%s "
