@@ -35,7 +35,6 @@
 #include "constants.h"
 #include "md.h"
 #include "methodcalls.h"
-#include "callKaffeException.h"
 #include "classMethod.h"
 #include "code.h"
 #include "exception.h"
@@ -68,12 +67,7 @@ vmExcept_getSyncObj(VmExceptHandler* eh)
 }
 
 #define FRAMEOBJECT(O, F, E)    (O) = vmExcept_getSyncObj((VmExceptHandler*)(F))
-#else
-
-#define DISPATCH_EXCEPTION(F,H,E) thread_data->exceptObj = NULL;\
-                                  CALL_KAFFE_EXCEPTION((F),(H),(E));
-
-#endif	/* TRANSLATOR */
+#endif	/* INTERPRETER */
 
 static void nullException(struct _exceptionFrame *);
 static void floatingException(struct _exceptionFrame *);
@@ -408,7 +402,7 @@ dispatchException(Hjava_lang_Throwable* eobj, stackTraceInfo* baseFrame)
 	     lastJniFrame = lastJniFrame->prev);
 
 	DBG(ELOOKUP,
-		dprintf ("dispatchException(): lastJniFrame is %p, fp %p\n", lastJniFrame, (lastJniFrame?lastJniFrame->frame.jni.fp:0)); );
+		dprintf ("dispatchException(): lastJniFrame is %p, fp %p\n", lastJniFrame, (lastJniFrame?(void*)lastJniFrame->frame.jni.fp:0)); );
 
 	/*
 	 * now walk up the stack 
@@ -421,7 +415,10 @@ dispatchException(Hjava_lang_Throwable* eobj, stackTraceInfo* baseFrame)
 		/*
 		 * if we reach the last jni frame, we're done
 		 */
-		if (lastJniFrame && vmExcept_JNIContains(lastJniFrame, frame->fp)) {
+		if (lastJniFrame 
+		    && vmExcept_JNIContains(lastJniFrame, 
+					    (JNIFrameAddress) frame->fp)) 
+		{
 			thread_data->exceptPtr = lastJniFrame;
 			vmExcept_jumpToHandler(lastJniFrame); /* doesn't return */
 		}
