@@ -23,16 +23,16 @@
 #if !defined(LIBRARYPATH)
 #define	LIBRARYPATH	"KAFFELIBRARYPATH"
 #endif
-#define	NATIVELIBRARY	"libnative"
+#define	NATIVELIBRARY	"libjavalang"
 
-struct _methods;
+struct _jmethodID;
 struct _errorInfo;
 
 void	initNative(void);
-int	loadNativeLibrary(char*, struct Hjava_lang_ClassLoader*, char*, size_t);
+int	loadNativeLibrary(const char*, struct Hjava_lang_ClassLoader*, char*, size_t);
 void	unloadNativeLibraries(struct Hjava_lang_ClassLoader*);
 void*	loadNativeLibrarySym(const char*);
-nativecode*	native(struct _methods*, struct _errorInfo*);
+nativecode*	native(struct _jmethodID*, struct _errorInfo*);
 void	addNativeFunc(char*, void*);
 char*	getLibraryPath(void);
 
