@@ -11,6 +11,8 @@
 #ifndef __machine_h
 #define	__machine_h
 
+#include "gtypes.h"
+#include "object.h"
 #include "threadData.h"
 
 #define current_class()		(meth->class)
@@ -44,6 +46,7 @@
   if (getMethodSignatureClass(idx, meth->class, true, 2, &cinfo, &einfo) \
         == false) { throwError(&einfo); }
 
+#define current_method()        (meth)
 #define	method_name()		(cinfo.name)
 #define	method_sig()		(cinfo.signature)
 #define method_idx()		(cinfo.method->idx)
@@ -110,6 +113,9 @@
 
 struct _jmethodID;
 struct _slots;
+
+typedef nativecode jitCodeHeader;
+
 void virtualMachine(struct _jmethodID*, struct _slots*, struct _slots*, threadData*); 
 void setupExceptionHandling(VmExceptHandler* eh, struct _jmethodID* meth, struct Hjava_lang_Object* syncobj, threadData*);
 void cleanupExceptionHandling(VmExceptHandler* eh, threadData*); 
