@@ -125,7 +125,18 @@ Kaffe_wrapper(Method* xmeth, void* func, bool use_JNI)
 #if defined(HAVE_FAKE_CALLS)
 	initFakeCalls();
 #endif
+
+	/* Do any machine dependent JIT initialization */
+#if defined(INIT_JIT_MD)
+	INIT_JIT_MD(xmeth);
+#endif
+
+#if defined(JIT3)
+	success = initInsnSequence(maxLocal, maxStack, &info);
+#else
 	success = initInsnSequence(xmeth, 0, maxLocal, maxStack, &info);
+#endif /* defined(JIT3) */
+
 	if (!success) {
 		goto done;
 	}
@@ -456,7 +467,7 @@ Kaffe_wrapper(Method* xmeth, void* func, bool use_JNI)
 	installMethodCode(NULL, xmeth, &ncode);
 
 	if (use_JNI)
-		xmeth->accflags |= ACC_JNI;
+		xmeth->kFlags |= KFLAG_JNI;
 
 	goto done;
 exitOOM:
@@ -490,5 +501,5 @@ done:
 void
 engine_create_wrapper (Method *meth, void *func)
 {
-	Kaffe_wrapper (meth, func, (meth->accflags&ACC_JNI)!=0);
+	Kaffe_wrapper (meth, func, (meth->kFlags & KFLAG_JNI)!=0);
 }
