@@ -196,6 +196,10 @@ extern void HAVE_cvt_double_float (sequence* s);
 extern void HAVE_cvt_double_int (sequence* s);
 #endif /* defined(HAVE_cvt_double_int) */
 
+#if defined(HAVE_cvt_double_int_ieee)
+extern void HAVE_cvt_double_int_ieee (sequence* s);
+#endif /* defined(HAVE_cvt_double_int_ieee) */
+
 #if defined(HAVE_cvt_double_long)
 extern void HAVE_cvt_double_long (sequence* s);
 #endif /* defined(HAVE_cvt_double_long) */
@@ -212,6 +216,10 @@ extern void HAVE_cvt_float_int (sequence* s);
 extern void HAVE_cvt_float_long (sequence* s);
 #endif /* defined(HAVE_cvt_float_long) */
 
+#if defined(HAVE_cvt_float_int_ieee)
+extern void HAVE_cvt_float_int_ieee (sequence* s);
+#endif /* defined(HAVE_cvt_float_int_ieee) */
+
 #if defined(HAVE_cvt_int_byte)
 extern void HAVE_cvt_int_byte (sequence* s);
 #endif /* defined(HAVE_cvt_int_byte) */
@@ -534,6 +542,14 @@ extern void HAVE_move_register_int (int toreg, int fromreg);
 extern void HAVE_move_register_ref (int toreg, int fromreg);
 #endif /* defined(HAVE_move_register_ref) */
 
+#if defined(HAVE_move_register_float)
+extern void HAVE_move_register_float (int toreg, int fromreg);
+#endif /* defined(HAVE_move_register_float) */
+
+#if defined(HAVE_move_register_double)
+extern void HAVE_move_register_double (int toreg, int fromreg);
+#endif /* defined(HAVE_move_register_double) */
+
 #if defined(HAVE_mul_double)
 extern void HAVE_mul_double (sequence* s);
 #endif /* defined(HAVE_mul_double) */
@@ -634,14 +650,6 @@ extern void HAVE_pusharg_ref_const (sequence* s);
 extern void HAVE_push_frame (sequence* s);
 #endif /* defined(HAVE_push_frame) */
 
-#if defined(HAVE_ref_eq_int)
-extern void HAVE_ref_eq_int (sequence* s);
-#endif /* defined(HAVE_ref_eq_int) */
-
-#if defined(HAVE_ref_eq_long)
-extern void HAVE_ref_eq_long (sequence* s);
-#endif /* defined(HAVE_ref_eq_long) */
-
 #if defined(HAVE_reload_double)
 extern void HAVE_reload_double (sequence* s);
 #endif /* defined(HAVE_reload_double) */
