@@ -153,6 +153,36 @@ jboolean KaffeJIT3_setupExitWithOOM(struct _errorInfo* einfo)
   return true;
 }
 
+/*
+ * Init instruction generation.
+ */
+jboolean
+initInsnSequence(int localsz, int stacksz, errorInfo* einfo)
+{
+	/* Clear various counters */
+	tmpslot = 0;
+	maxTemp = 0;
+	maxPush = 0;
+	stackno = localsz + stacksz;
+	npc = 0;
+
+	initSeq();
+	initRegisters();
+	initSlots(stackno);
+
+	/* Before generating code, try to guess how much space we'll need. */
+	codeblock_size = ALLOCCODEBLOCKSZ;
+	codeblock = gc_malloc(codeblock_size + CODEBLOCKREDZONE,
+			      KGC_ALLOC_JIT_CODEBLOCK);
+	if (codeblock == 0) {
+		postOutOfMemory(einfo);
+		return (false);
+	}
+	CODEPC = 0;
+	
+	return (true);
+}
+
 /*
  * Translate a method into native code.
  *
@@ -312,7 +342,12 @@ DBG(MOREJIT,
 	 * Initialise the translator.
 	 */
 	initFakeCalls();
-	success = initInsnSequence(xmeth, codeperbytecode * METHOD_BYTECODE_LEN(xmeth), xmeth->localsz, xmeth->stacksz, einfo);
+
+	/* Do any machine dependent JIT initialization */
+#if defined(INIT_JIT_MD)
+	INIT_JIT_MD(xmeth);
+#endif
+	success = initInsnSequence(xmeth->localsz, xmeth->stacksz, einfo);
 	if (success == false) {
 		goto done;
 	}
@@ -613,7 +648,7 @@ installMethodCode(void* ignore UNUSED, Method* meth, nativeCodeInfo* code)
 
 	SET_METHOD_JITCODE(meth, code->code);
 
-	meth->c.ncode.ncode_start = code->mem;
+	setMethodCodeStart(meth, code->mem);
 	meth->c.ncode.ncode_end = (void*)((uintp)code->code + code->codelen);
 	
 #if defined(KAFFE_FEEDBACK)
@@ -801,41 +836,6 @@ installMethodCode(void* ignore UNUSED, Method* meth, nativeCodeInfo* code)
 #endif
 }
 
-/*
- * Init instruction generation.
- */
-jboolean
-initInsnSequence(Method* meth, int codesize UNUSED, int localsz, int stacksz, errorInfo* einfo)
-{
-	/* Clear various counters */
-	tmpslot = 0;
-	maxTemp = 0;
-	maxPush = 0;
-	stackno = localsz + stacksz;
-	npc = 0;
-
-	/* Do any machine dependent JIT initialization */
-#if defined(INIT_JIT_MD)
-	INIT_JIT_MD(meth);
-#endif
-
-	initSeq();
-	initRegisters();
-	initSlots(stackno);
-
-	/* Before generating code, try to guess how much space we'll need. */
-	codeblock_size = ALLOCCODEBLOCKSZ;
-	codeblock = gc_malloc(codeblock_size + CODEBLOCKREDZONE,
-			      KGC_ALLOC_JIT_CODEBLOCK);
-	if (codeblock == 0) {
-		postOutOfMemory(einfo);
-		return (false);
-	}
-	CODEPC = 0;
-	
-	return (true);
-}
-
 /*
  * Generate instructions from current set of sequences.
  */
@@ -1421,7 +1421,7 @@ profilerClassStat(Hjava_lang_Class *clazz, void *param UNUSED)
 	Method *meth;
 	int mindex;
 
-	for (mindex = CLASS_NMETHODS(clazz), meth = CLASS_METHODS(clazz); mindex-- > 0; meth++) {
+	for (mindex = CLASS_NMETHODS(clazz), meth = Kaffe_get_class_methods(clazz); mindex-- > 0; meth++) {
 		if (meth->callsCount == 0)
 			continue;
 
