@@ -14,6 +14,7 @@
 #define __machine_h
 
 #include "gtypes.h"
+#include "object.h"
 
 /* -------------------------------------------------------------------- */
 
@@ -61,6 +62,7 @@
 #define	get_interface_method_info(IDX) \
 	if (getMethodSignatureClass(IDX, xmeth->class, true, 2, &cinfo, einfo) == false) { success = false; goto done; }
 
+#define current_method()  (xmeth)
 #define	method_name()	(cinfo.name)
 #define	method_sig()	(cinfo.signature)
 #define	method_idx()	(cinfo.method->idx)
@@ -194,7 +196,7 @@ struct SlotData;
 struct _label_;
 struct _sequence;
 struct _errorInfo;
-extern jboolean initInsnSequence(Method *meth, int codesize, int localsz, int stacksz, struct _errorInfo *einfo);
+extern jboolean initInsnSequence(int localsz, int stacksz, struct _errorInfo *einfo);
 extern jboolean finishInsnSequence(void*, nativeCodeInfo*, struct _errorInfo*);
 extern void     installMethodCode(void*, Method*, nativeCodeInfo*);
 #define HAVE_FAKE_CALLS 1
