@@ -2,10 +2,10 @@
  * jni-arrays.c
  * Java Native Interface - Array handling JNI functions.
  *
- * Copyright (c) 1996, 1997, 2004
+ * Copyright (c) 1996, 1997
  *	Transvirtual Technologies, Inc.  All rights reserved.
  *
- * Copyright (c) 2004
+ * Copyright (c) 2004, 2005
  *      The Kaffe.org's developers. See ChangeLog for details.
  *
  * See the file "license.terms" for information on usage and redistribution 
@@ -13,6 +13,11 @@
  */
 
 #include "config.h"
+
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
+
 #include "jni_i.h"
 #include "jni.h"
 #include "jni_funcs.h"
@@ -25,13 +30,15 @@ jobject
 KaffeJNI_GetObjectArrayElement(JNIEnv* env UNUSED, jobjectArray arr, jsize elem)
 {
   jobject obj;
+  jobjectArray arr_local;
 
   BEGIN_EXCEPTION_HANDLING(NULL);
 
-  if (elem >= (jsize)obj_length((HArrayOfObject*)arr)) {
+  arr_local = unveil(arr);
+  if (elem >= (jsize)obj_length((HArrayOfObject*)arr_local)) {
     throwException(ArrayIndexOutOfBoundsException);
   }
-  obj = unhand_array((HArrayOfObject*)arr)->body[elem];
+  obj = unhand_array((HArrayOfObject*)arr_local)->body[elem];
 
   ADD_REF(obj);
   END_EXCEPTION_HANDLING();
@@ -41,12 +48,18 @@ KaffeJNI_GetObjectArrayElement(JNIEnv* env UNUSED, jobjectArray arr, jsize elem)
 void
 KaffeJNI_SetObjectArrayElement(JNIEnv* env UNUSED, jobjectArray arr, jsize elem, jobject val)
 {
+  jobjectArray arr_local;
+  jobject val_local;
   BEGIN_EXCEPTION_HANDLING_VOID();
 
-  if (elem >= (jsize)obj_length((HArrayOfObject*)arr)) {
+  arr_local = unveil(arr);
+  val_local = unveil(val);
+
+  if (elem >= (jsize)obj_length((HArrayOfObject*)arr_local)) {
     throwException(ArrayIndexOutOfBoundsException);
   }
-  unhand_array((HArrayOfObject*)arr)->body[elem] = (Hjava_lang_Object*)val;
+  unhand_array((HArrayOfObject*)arr_local)->body[elem] =
+    (Hjava_lang_Object*)val_local;
 
   END_EXCEPTION_HANDLING();
 }
@@ -56,14 +69,18 @@ KaffeJNI_NewObjectArray(JNIEnv* env UNUSED, jsize len, jclass cls, jobject init)
 {
   HArrayOfObject* obj;
   jsize i;
+  jclass cls_local;
+  jobject init_local;
 
   BEGIN_EXCEPTION_HANDLING(NULL);
 
-  obj = (HArrayOfObject*)newArray((Hjava_lang_Class*)cls, len);
+  cls_local = unveil(cls);
+  init_local = unveil(init);
+  obj = (HArrayOfObject*)newArray((Hjava_lang_Class*)cls_local, len);
 
   /* I assume this is what init is for */
   for (i = 0; i < len; i++) {
-    unhand_array(obj)->body[i] = (Hjava_lang_Object*)init;
+    unhand_array(obj)->body[i] = (Hjava_lang_Object*)init_local;
   }
 
   ADD_REF(obj);
@@ -184,148 +201,170 @@ KaffeJNI_NewDoubleArray(JNIEnv* env UNUSED, jsize len)
 }
 
 jboolean*
-KaffeJNI_GetBooleanArrayElements(JNIEnv* env UNUSED, jbooleanArray arr, jbool* iscopy)
+KaffeJNI_GetBooleanArrayElements(JNIEnv* env UNUSED, jbooleanArray arr, jboolean* iscopy)
 {
   jboolean* array;
+  jbooleanArray arr_local;
   BEGIN_EXCEPTION_HANDLING(NULL);
 
+  arr_local = unveil(arr);
   if (iscopy != NULL) {
     *iscopy = JNI_FALSE;
   }
-  array = unhand_array((HArrayOfBoolean*)arr)->body;
+  array = unhand_array((HArrayOfBoolean*)arr_local)->body;
 
   END_EXCEPTION_HANDLING();
   return (array);
 }
 
 jbyte*
-KaffeJNI_GetByteArrayElements(JNIEnv* env UNUSED, jbyteArray arr, jbool* iscopy)
+KaffeJNI_GetByteArrayElements(JNIEnv* env UNUSED, jbyteArray arr, jboolean* iscopy)
 {
   jbyte* array;
+  jbyteArray arr_local;
   BEGIN_EXCEPTION_HANDLING(NULL);
 
+  arr_local = unveil(arr);
   if (iscopy != NULL) {
     *iscopy = JNI_FALSE;
   }
-  array = unhand_array((HArrayOfByte*)arr)->body;
+  array = unhand_array((HArrayOfByte*)arr_local)->body;
 
   END_EXCEPTION_HANDLING();
   return (array);
 }
 
 void*
-KaffeJNI_GetPrimitiveArrayCritical(JNIEnv* env, jarray arr, jbool* iscopy)
+KaffeJNI_GetPrimitiveArrayCritical(JNIEnv* env, jarray arr, jboolean* iscopy)
 {
   return (KaffeJNI_GetByteArrayElements(env, (jbyteArray)arr, iscopy));
 }
 
 jchar*
-KaffeJNI_GetCharArrayElements(JNIEnv* env UNUSED, jcharArray arr, jbool* iscopy)
+KaffeJNI_GetCharArrayElements(JNIEnv* env UNUSED, jcharArray arr, jboolean* iscopy)
 {
   jchar* array;
+  jcharArray arr_local;
   BEGIN_EXCEPTION_HANDLING(NULL);
-
+  
+  arr_local = unveil(arr);
   if (iscopy != NULL) {
     *iscopy = JNI_FALSE;
   }
-  array = unhand_array((HArrayOfChar*)arr)->body;
+  array = unhand_array((HArrayOfChar*)arr_local)->body;
 
   END_EXCEPTION_HANDLING();
   return (array);
 }
 
 jshort*
-KaffeJNI_GetShortArrayElements(JNIEnv* env UNUSED, jshortArray arr, jbool* iscopy)
+KaffeJNI_GetShortArrayElements(JNIEnv* env UNUSED, jshortArray arr, jboolean* iscopy)
 {
   jshort* array;
+  jshortArray arr_local;
   BEGIN_EXCEPTION_HANDLING(NULL);
 
+  arr_local = unveil(arr);
   if (iscopy != NULL) {
     *iscopy = JNI_FALSE;
   }
-  array = unhand_array((HArrayOfShort*)arr)->body;
+  array = unhand_array((HArrayOfShort*)arr_local)->body;
 
   END_EXCEPTION_HANDLING();
   return (array);
 }
 
 jint*
-KaffeJNI_GetIntArrayElements(JNIEnv* env UNUSED, jintArray arr, jbool* iscopy)
+KaffeJNI_GetIntArrayElements(JNIEnv* env UNUSED, jintArray arr, jboolean* iscopy)
 {
   jint* array;
+  jintArray arr_local;
   BEGIN_EXCEPTION_HANDLING(NULL);
 
+  arr_local = unveil(arr);
   if (iscopy != NULL) {
     *iscopy = JNI_FALSE;
   }
-  array = unhand_array((HArrayOfInt*)arr)->body;
+  array = unhand_array((HArrayOfInt*)arr_local)->body;
 
   END_EXCEPTION_HANDLING();
   return (array);
 }
 
 jlong*
-KaffeJNI_GetLongArrayElements(JNIEnv* env UNUSED, jlongArray arr, jbool* iscopy)
+KaffeJNI_GetLongArrayElements(JNIEnv* env UNUSED, jlongArray arr, jboolean* iscopy)
 {
   jlong* array;
+  jlongArray arr_local;
   BEGIN_EXCEPTION_HANDLING(NULL);
 
+  arr_local = unveil(arr);
   if (iscopy != NULL) {
     *iscopy = JNI_FALSE;
   }
-  array = unhand_array((HArrayOfLong*)arr)->body;
+  array = unhand_array((HArrayOfLong*)arr_local)->body;
 
   END_EXCEPTION_HANDLING();
   return (array);
 }
 
 jfloat*
-KaffeJNI_GetFloatArrayElements(JNIEnv* env UNUSED, jfloatArray arr, jbool* iscopy)
+KaffeJNI_GetFloatArrayElements(JNIEnv* env UNUSED, jfloatArray arr, jboolean* iscopy)
 {
   jfloat* array;
+  jfloatArray arr_local;
   BEGIN_EXCEPTION_HANDLING(NULL);
 
+  arr_local = unveil(arr);
   if (iscopy != NULL) {
     *iscopy = JNI_FALSE;
   }
-  array = unhand_array((HArrayOfFloat*)arr)->body;
+  array = unhand_array((HArrayOfFloat*)arr_local)->body;
 
   END_EXCEPTION_HANDLING();
   return (array);
 }
 
 jdouble*
-KaffeJNI_GetDoubleArrayElements(JNIEnv* env UNUSED, jdoubleArray arr, jbool* iscopy)
+KaffeJNI_GetDoubleArrayElements(JNIEnv* env UNUSED, jdoubleArray arr, jboolean* iscopy)
 {
   jdouble* array;
+  jdoubleArray arr_local;
   BEGIN_EXCEPTION_HANDLING(NULL);
 
+  arr_local = unveil(arr);
   if (iscopy != NULL) {
     *iscopy = JNI_FALSE;
   }
-  array = unhand_array((HArrayOfDouble*)arr)->body;
+  array = unhand_array((HArrayOfDouble*)arr_local)->body;
 
   END_EXCEPTION_HANDLING();
   return (array);
 }
 
 void
-KaffeJNI_ReleaseBooleanArrayElements(JNIEnv* env UNUSED, jbooleanArray arr, jbool* elems, jint mode)
+KaffeJNI_ReleaseBooleanArrayElements(JNIEnv* env UNUSED, jbooleanArray arr, jboolean* elems, jint mode)
 {
+  jbooleanArray arr_local;
   BEGIN_EXCEPTION_HANDLING_VOID();
 
-  if (elems != unhand_array((HArrayOfBoolean*)arr)->body) {
+  arr_local = unveil(arr);
+  if (elems != unhand_array((HArrayOfBoolean*)arr_local)->body) {
     switch (mode) {
     case JNI_COMMIT:
-      memcpy(unhand_array((HArrayOfBoolean*)arr)->body, elems, obj_length((HArrayOfBoolean*)arr) * sizeof(jboolean));
+      memcpy(unhand_array((HArrayOfBoolean*)arr_local)->body, elems,
+	     obj_length((HArrayOfBoolean*)arr_local) * sizeof(jboolean));
       break;
     case 0:
-      memcpy(unhand_array((HArrayOfBoolean*)arr)->body, elems, obj_length((HArrayOfBoolean*)arr) * sizeof(jboolean));
+      memcpy(unhand_array((HArrayOfBoolean*)arr_local)->body, elems,
+	     obj_length((HArrayOfBoolean*)arr_local) * sizeof(jboolean));
       KFREE(elems);
       break;
     case JNI_ABORT:
       KFREE(elems);
       break;
+    default:
+      break;
     }
   }
   END_EXCEPTION_HANDLING();
@@ -334,20 +373,26 @@ KaffeJNI_ReleaseBooleanArrayElements(JNIEnv* env UNUSED, jbooleanArray arr, jboo
 void
 KaffeJNI_ReleaseByteArrayElements(JNIEnv* env UNUSED, jbyteArray arr, jbyte* elems, jint mode)
 {
+  jbyteArray arr_local;
   BEGIN_EXCEPTION_HANDLING_VOID();
 
-  if (elems != unhand_array((HArrayOfByte*)arr)->body) {
+  arr_local = unveil(arr);
+  if (elems != unhand_array((HArrayOfByte*)arr_local)->body) {
     switch (mode) {
     case JNI_COMMIT:
-      memcpy(unhand_array((HArrayOfByte*)arr)->body, elems, obj_length((HArrayOfByte*)arr) * sizeof(jbyte));
+      memcpy(unhand_array((HArrayOfByte*)arr_local)->body, elems,
+	     obj_length((HArrayOfByte*)arr_local) * sizeof(jbyte));
       break;
     case 0:
-      memcpy(unhand_array((HArrayOfByte*)arr)->body, elems, obj_length((HArrayOfByte*)arr) * sizeof(jbyte));
+      memcpy(unhand_array((HArrayOfByte*)arr_local)->body, elems,
+	     obj_length((HArrayOfByte*)arr_local) * sizeof(jbyte));
       KFREE(elems);
       break;
     case JNI_ABORT:
       KFREE(elems);
       break;
+    default:
+      break;
     }
   }
   END_EXCEPTION_HANDLING();
@@ -362,20 +407,26 @@ KaffeJNI_ReleasePrimitiveArrayCritical(JNIEnv* env, jbyteArray arr, void* elems,
 void
 KaffeJNI_ReleaseCharArrayElements(JNIEnv* env UNUSED, jcharArray arr, jchar* elems, jint mode)
 {
+  jcharArray arr_local;
   BEGIN_EXCEPTION_HANDLING_VOID();
 
-  if (elems != unhand_array((HArrayOfChar*)arr)->body) {
+  arr_local = unveil(arr);
+  if (elems != unhand_array((HArrayOfChar*)arr_local)->body) {
     switch (mode) {
     case JNI_COMMIT:
-      memcpy(unhand_array((HArrayOfChar*)arr)->body, elems, obj_length((HArrayOfChar*)arr) * sizeof(jchar));
+      memcpy(unhand_array((HArrayOfChar*)arr_local)->body, elems,
+	     obj_length((HArrayOfChar*)arr_local) * sizeof(jchar));
       break;
     case 0:
-      memcpy(unhand_array((HArrayOfChar*)arr)->body, elems, obj_length((HArrayOfChar*)arr) * sizeof(jchar));
+      memcpy(unhand_array((HArrayOfChar*)arr_local)->body, elems,
+	     obj_length((HArrayOfChar*)arr_local) * sizeof(jchar));
       KFREE(elems);
       break;
     case JNI_ABORT:
       KFREE(elems);
       break;
+    default:
+      break;
     }
   }
   END_EXCEPTION_HANDLING();
@@ -384,20 +435,26 @@ KaffeJNI_ReleaseCharArrayElements(JNIEnv* env UNUSED, jcharArray arr, jchar* ele
 void
 KaffeJNI_ReleaseShortArrayElements(JNIEnv* env UNUSED, jshortArray arr, jshort* elems, jint mode)
 {
+  jshortArray arr_local;
   BEGIN_EXCEPTION_HANDLING_VOID();
 
-  if (elems != unhand_array((HArrayOfShort*)arr)->body) {
+  arr_local = unveil(arr);
+  if (elems != unhand_array((HArrayOfShort*)arr_local)->body) {
     switch (mode) {
     case JNI_COMMIT:
-      memcpy(unhand_array((HArrayOfShort*)arr)->body, elems, obj_length((HArrayOfShort*)arr) * sizeof(jshort));
+      memcpy(unhand_array((HArrayOfShort*)arr_local)->body, elems,
+	     obj_length((HArrayOfShort*)arr_local) * sizeof(jshort));
       break;
     case 0:
-      memcpy(unhand_array((HArrayOfShort*)arr)->body, elems, obj_length((HArrayOfShort*)arr) * sizeof(jshort));
+      memcpy(unhand_array((HArrayOfShort*)arr_local)->body, elems,
+	     obj_length((HArrayOfShort*)arr_local) * sizeof(jshort));
       KFREE(elems);
       break;
     case JNI_ABORT:
       KFREE(elems);
       break;
+    default:
+      break;
     }
   }
   END_EXCEPTION_HANDLING();
@@ -406,20 +463,27 @@ KaffeJNI_ReleaseShortArrayElements(JNIEnv* env UNUSED, jshortArray arr, jshort*
 void
 KaffeJNI_ReleaseIntArrayElements(JNIEnv* env UNUSED, jintArray arr, jint* elems, jint mode)
 {
+  jintArray arr_local;
   BEGIN_EXCEPTION_HANDLING_VOID();
 
-  if (elems != unhand_array((HArrayOfInt*)arr)->body) {
+  arr_local = unveil(arr);
+  if (elems != unhand_array((HArrayOfInt*)arr_local)->body) {
     switch (mode) {
     case JNI_COMMIT:
-      memcpy(unhand_array((HArrayOfInt*)arr)->body, elems, obj_length((HArrayOfInt*)arr) * sizeof(jint));
+      memcpy(unhand_array((HArrayOfInt*)arr_local)->body, elems,
+	     obj_length((HArrayOfInt*)arr_local) * sizeof(jint));
       break;
     case 0:
-      memcpy(unhand_array((HArrayOfInt*)arr)->body, elems, obj_length((HArrayOfInt*)arr) * sizeof(jint));
+      memcpy(unhand_array((HArrayOfInt*)arr_local)->body, elems,
+	     obj_length((HArrayOfInt*)arr_local) * sizeof(jint));
       KFREE(elems);
       break;
     case JNI_ABORT:
+
       KFREE(elems);
       break;
+    default:
+      break;
     }
   }
   END_EXCEPTION_HANDLING();
@@ -428,20 +492,26 @@ KaffeJNI_ReleaseIntArrayElements(JNIEnv* env UNUSED, jintArray arr, jint* elems,
 void
 KaffeJNI_ReleaseLongArrayElements(JNIEnv* env UNUSED, jlongArray arr, jlong* elems, jint mode)
 {
+  jlongArray arr_local;
   BEGIN_EXCEPTION_HANDLING_VOID();
 
-  if (elems != unhand_array((HArrayOfLong*)arr)->body) {
+  arr_local = unveil(arr);
+  if (elems != unhand_array((HArrayOfLong*)arr_local)->body) {
     switch (mode) {
     case JNI_COMMIT:
-      memcpy(unhand_array((HArrayOfLong*)arr)->body, elems, obj_length((HArrayOfLong*)arr) * sizeof(jlong));
+      memcpy(unhand_array((HArrayOfLong*)arr_local)->body, elems,
+	     obj_length((HArrayOfLong*)arr_local) * sizeof(jlong));
       break;
     case 0:
-      memcpy(unhand_array((HArrayOfLong*)arr)->body, elems, obj_length((HArrayOfLong*)arr) * sizeof(jlong));
+      memcpy(unhand_array((HArrayOfLong*)arr_local)->body, elems,
+	     obj_length((HArrayOfLong*)arr_local) * sizeof(jlong));
       KFREE(elems);
       break;
     case JNI_ABORT:
       KFREE(elems);
       break;
+    default:
+      break;
     }
   }
   END_EXCEPTION_HANDLING();
@@ -450,20 +520,26 @@ KaffeJNI_ReleaseLongArrayElements(JNIEnv* env UNUSED, jlongArray arr, jlong* ele
 void
 KaffeJNI_ReleaseFloatArrayElements(JNIEnv* env UNUSED, jfloatArray arr, jfloat* elems, jint mode)
 {
+  jfloatArray arr_local;
   BEGIN_EXCEPTION_HANDLING_VOID();
 
-  if (elems != unhand_array((HArrayOfFloat*)arr)->body) {
+  arr_local = unveil(arr);
+  if (elems != unhand_array((HArrayOfFloat*)arr_local)->body) {
     switch (mode) {
     case JNI_COMMIT:
-      memcpy(unhand_array((HArrayOfFloat*)arr)->body, elems, obj_length((HArrayOfFloat*)arr) * sizeof(jfloat));
+      memcpy(unhand_array((HArrayOfFloat*)arr_local)->body, elems,
+	     obj_length((HArrayOfFloat*)arr_local) * sizeof(jfloat));
       break;
     case 0:
-      memcpy(unhand_array((HArrayOfFloat*)arr)->body, elems, obj_length((HArrayOfFloat*)arr) * sizeof(jfloat));
+      memcpy(unhand_array((HArrayOfFloat*)arr_local)->body, elems,
+	     obj_length((HArrayOfFloat*)arr_local) * sizeof(jfloat));
       KFREE(elems);
       break;
     case JNI_ABORT:
       KFREE(elems);
       break;
+    default:
+      break;
     }
   }
   END_EXCEPTION_HANDLING();
@@ -472,34 +548,44 @@ KaffeJNI_ReleaseFloatArrayElements(JNIEnv* env UNUSED, jfloatArray arr, jfloat*
 void
 KaffeJNI_ReleaseDoubleArrayElements(JNIEnv* env UNUSED, jdoubleArray arr, jdouble* elems, jint mode)
 {
+  jdoubleArray arr_local;
   BEGIN_EXCEPTION_HANDLING_VOID();
 
-  if (elems != unhand_array((HArrayOfDouble*)arr)->body) {
+  arr_local = unveil(arr);
+  if (elems != unhand_array((HArrayOfDouble*)arr_local)->body) {
     switch (mode) {
     case JNI_COMMIT:
-      memcpy(unhand_array((HArrayOfDouble*)arr)->body, elems, obj_length((HArrayOfDouble*)arr) * sizeof(jdouble));
+      memcpy(unhand_array((HArrayOfDouble*)arr_local)->body, elems,
+	     obj_length((HArrayOfDouble*)arr_local) * sizeof(jdouble));
       break;
     case 0:
-      memcpy(unhand_array((HArrayOfDouble*)arr)->body, elems, obj_length((HArrayOfDouble*)arr) * sizeof(jdouble));
+      memcpy(unhand_array((HArrayOfDouble*)arr_local)->body, elems,
+	     obj_length((HArrayOfDouble*)arr_local) * sizeof(jdouble));
       KFREE(elems);
       break;
     case JNI_ABORT:
       KFREE(elems);
       break;
+    default:
+      break;
     }
   }
   END_EXCEPTION_HANDLING();
 }
 
 void
-KaffeJNI_GetBooleanArrayRegion(JNIEnv* env UNUSED, jbooleanArray arr, jsize start, jsize len, jbool* data)
+KaffeJNI_GetBooleanArrayRegion(JNIEnv* env UNUSED, jbooleanArray arr, jsize start, jsize len, jboolean* data)
 {
+  jbooleanArray arr_local;
   BEGIN_EXCEPTION_HANDLING_VOID();
 
-  if (start >= obj_length((HArrayOfBoolean*)arr) || start + len > obj_length((HArrayOfBoolean*)arr)) {
+  arr_local = unveil(arr);
+  if (start >= obj_length((HArrayOfBoolean*)arr_local)
+      || start + len > obj_length((HArrayOfBoolean*)arr_local)) {
     throwException(ArrayIndexOutOfBoundsException);
   }
-  memcpy(data, &unhand_array((HArrayOfBoolean*)arr)->body[start], len * sizeof(jboolean));
+  memcpy(data, &unhand_array((HArrayOfBoolean*)arr_local)->body[start],
+	 len * sizeof(jboolean));
 
   END_EXCEPTION_HANDLING();
 }
@@ -507,12 +593,16 @@ KaffeJNI_GetBooleanArrayRegion(JNIEnv* env UNUSED, jbooleanArray arr, jsize star
 void
 KaffeJNI_GetByteArrayRegion(JNIEnv* env UNUSED, jbyteArray arr, jsize start, jsize len, jbyte* data)
 {
+  jbyteArray arr_local;
   BEGIN_EXCEPTION_HANDLING_VOID();
 
-  if (start >= obj_length((HArrayOfByte*)arr) || start + len > obj_length((HArrayOfByte*)arr)) {
+  arr_local = unveil(arr);
+  if (start >= obj_length((HArrayOfByte*)arr_local)
+      || start + len > obj_length((HArrayOfByte*)arr_local)) {
     throwException(ArrayIndexOutOfBoundsException);
   }
-  memcpy(data, &unhand_array((HArrayOfByte*)arr)->body[start], len * sizeof(jbyte));
+  memcpy(data, &unhand_array((HArrayOfByte*)arr_local)->body[start],
+	 len * sizeof(jbyte));
 
   END_EXCEPTION_HANDLING();
 }
@@ -520,12 +610,16 @@ KaffeJNI_GetByteArrayRegion(JNIEnv* env UNUSED, jbyteArray arr, jsize start, jsi
 void
 KaffeJNI_GetCharArrayRegion(JNIEnv* env UNUSED, jcharArray arr, jsize start, jsize len, jchar* data)
 {
+  jcharArray arr_local;
   BEGIN_EXCEPTION_HANDLING_VOID();
 
-  if (start >= obj_length((HArrayOfChar*)arr) || start + len > obj_length((HArrayOfChar*)arr)) {
+  arr_local = unveil(arr);
+  if (start >= obj_length((HArrayOfChar*)arr_local)
+      || start + len > obj_length((HArrayOfChar*)arr_local)) {
     throwException(ArrayIndexOutOfBoundsException);
   }
-  memcpy(data, &unhand_array((HArrayOfChar*)arr)->body[start], len * sizeof(jchar));
+  memcpy(data, &unhand_array((HArrayOfChar*)arr_local)->body[start],
+	 len * sizeof(jchar));
 
   END_EXCEPTION_HANDLING();
 }
@@ -533,12 +627,16 @@ KaffeJNI_GetCharArrayRegion(JNIEnv* env UNUSED, jcharArray arr, jsize start, jsi
 void
 KaffeJNI_GetShortArrayRegion(JNIEnv* env UNUSED, jshortArray arr, jsize start, jsize len, jshort* data)
 {
+  jshortArray arr_local;
   BEGIN_EXCEPTION_HANDLING_VOID();
 
-  if (start >= obj_length((HArrayOfShort*)arr) || start + len > obj_length((HArrayOfShort*)arr)) {
+  arr_local = unveil(arr);
+  if (start >= obj_length((HArrayOfShort*)arr_local)
+      || start + len > obj_length((HArrayOfShort*)arr_local)) {
     throwException(ArrayIndexOutOfBoundsException);
   }
-  memcpy(data, &unhand_array((HArrayOfShort*)arr)->body[start], len * sizeof(jshort));
+  memcpy(data, &unhand_array((HArrayOfShort*)arr_local)->body[start],
+	 len * sizeof(jshort));
 
   END_EXCEPTION_HANDLING();
 }
@@ -546,12 +644,16 @@ KaffeJNI_GetShortArrayRegion(JNIEnv* env UNUSED, jshortArray arr, jsize start, j
 void
 KaffeJNI_GetIntArrayRegion(JNIEnv* env UNUSED, jintArray arr, jsize start, jsize len, jint* data)
 {
+  jintArray arr_local;
   BEGIN_EXCEPTION_HANDLING_VOID();
 
-  if (start >= obj_length((HArrayOfInt*)arr) || start + len > obj_length((HArrayOfInt*)arr)) {
+  arr_local = unveil(arr);
+  if (start >= obj_length((HArrayOfInt*)arr_local)
+      || start + len > obj_length((HArrayOfInt*)arr_local)) {
     throwException(ArrayIndexOutOfBoundsException);
   }
-  memcpy(data, &unhand_array((HArrayOfInt*)arr)->body[start], len * sizeof(jint));
+  memcpy(data, &unhand_array((HArrayOfInt*)arr_local)->body[start],
+	 len * sizeof(jint));
 
   END_EXCEPTION_HANDLING();
 }
@@ -559,12 +661,16 @@ KaffeJNI_GetIntArrayRegion(JNIEnv* env UNUSED, jintArray arr, jsize start, jsize
 void
 KaffeJNI_GetLongArrayRegion(JNIEnv* env UNUSED, jlongArray arr, jsize start, jsize len, jlong* data)
 {
+  jlongArray arr_local;
   BEGIN_EXCEPTION_HANDLING_VOID();
 
-  if (start >= obj_length((HArrayOfLong*)arr) || start + len > obj_length((HArrayOfLong*)arr)) {
+  arr_local = unveil(arr);
+  if (start >= obj_length((HArrayOfLong*)arr_local)
+      || start + len > obj_length((HArrayOfLong*)arr_local)) {
     throwException(ArrayIndexOutOfBoundsException);
   }
-  memcpy(data, &unhand_array((HArrayOfLong*)arr)->body[start], len * sizeof(jlong));
+  memcpy(data, &unhand_array((HArrayOfLong*)arr_local)->body[start],
+	 len * sizeof(jlong));
 
   END_EXCEPTION_HANDLING();
 }
@@ -572,12 +678,16 @@ KaffeJNI_GetLongArrayRegion(JNIEnv* env UNUSED, jlongArray arr, jsize start, jsi
 void
 KaffeJNI_GetFloatArrayRegion(JNIEnv* env UNUSED, jfloatArray arr, jsize start, jsize len, jfloat* data)
 {
+  jfloatArray arr_local;
   BEGIN_EXCEPTION_HANDLING_VOID();
 
-  if (start >= obj_length((HArrayOfFloat*)arr) || start + len > obj_length((HArrayOfFloat*)arr)) {
+  arr_local = unveil(arr);
+  if (start >= obj_length((HArrayOfFloat*)arr_local)
+      || start + len > obj_length((HArrayOfFloat*)arr_local)) {
     throwException(ArrayIndexOutOfBoundsException);
   }
-  memcpy(data, &unhand_array((HArrayOfFloat*)arr)->body[start], len * sizeof(jfloat));
+  memcpy(data, &unhand_array((HArrayOfFloat*)arr_local)->body[start],
+	 len * sizeof(jfloat));
 
   END_EXCEPTION_HANDLING();
 }
@@ -585,25 +695,33 @@ KaffeJNI_GetFloatArrayRegion(JNIEnv* env UNUSED, jfloatArray arr, jsize start, j
 void
 KaffeJNI_GetDoubleArrayRegion(JNIEnv* env UNUSED, jdoubleArray arr, jsize start, jsize len, jdouble* data)
 {
+  jdoubleArray arr_local;
   BEGIN_EXCEPTION_HANDLING_VOID();
 
-  if (start >= obj_length((HArrayOfDouble*)arr) || start + len > obj_length((HArrayOfDouble*)arr)) {
+  arr_local = unveil(arr);
+  if (start >= obj_length((HArrayOfDouble*)arr_local)
+      || start + len > obj_length((HArrayOfDouble*)arr_local)) {
     throwException(ArrayIndexOutOfBoundsException);
   }
-  memcpy(data, &unhand_array((HArrayOfDouble*)arr)->body[start], len * sizeof(jdouble));
+  memcpy(data, &unhand_array((HArrayOfDouble*)arr_local)->body[start],
+	 len * sizeof(jdouble));
 
   END_EXCEPTION_HANDLING();
 }
 
 void
-KaffeJNI_SetBooleanArrayRegion(JNIEnv* env UNUSED, jbooleanArray arr, jsize start, jsize len, jbool* data)
+KaffeJNI_SetBooleanArrayRegion(JNIEnv* env UNUSED, jbooleanArray arr, jsize start, jsize len, jboolean* data)
 {
+  jbooleanArray arr_local;
   BEGIN_EXCEPTION_HANDLING_VOID();
 
-  if (start >= obj_length((HArrayOfBoolean*)arr) || start+len > obj_length((HArrayOfBoolean*)arr)) {
+  arr_local = unveil(arr);
+  if (start >= obj_length((HArrayOfBoolean*)arr_local)
+      || start+len > obj_length((HArrayOfBoolean*)arr_local)) {
     throwException(ArrayIndexOutOfBoundsException);
   }
-  memcpy(&unhand_array((HArrayOfBoolean*)arr)->body[start], data, len * sizeof(jboolean));
+  memcpy(&unhand_array((HArrayOfBoolean*)arr_local)->body[start], data,
+	 len * sizeof(jboolean));
 
   END_EXCEPTION_HANDLING();
 }
@@ -611,12 +729,16 @@ KaffeJNI_SetBooleanArrayRegion(JNIEnv* env UNUSED, jbooleanArray arr, jsize star
 void
 KaffeJNI_SetByteArrayRegion(JNIEnv* env UNUSED, jbyteArray arr, jsize start, jsize len, jbyte* data)
 {
+  jbyteArray arr_local;
   BEGIN_EXCEPTION_HANDLING_VOID();
 
-  if (start >= obj_length((HArrayOfByte*)arr) || start+len > obj_length((HArrayOfByte*)arr)) {
+ arr_local = unveil(arr);
+  if (start >= obj_length((HArrayOfByte*)arr_local)
+      || start+len > obj_length((HArrayOfByte*)arr_local)) {
     throwException(ArrayIndexOutOfBoundsException);
   }
-  memcpy(&unhand_array((HArrayOfByte*)arr)->body[start], data, len * sizeof(jbyte));
+  memcpy(&unhand_array((HArrayOfByte*)arr_local)->body[start], data,
+	 len * sizeof(jbyte));
 
   END_EXCEPTION_HANDLING();
 }
@@ -624,12 +746,16 @@ KaffeJNI_SetByteArrayRegion(JNIEnv* env UNUSED, jbyteArray arr, jsize start, jsi
 void
 KaffeJNI_SetCharArrayRegion(JNIEnv* env UNUSED, jcharArray arr, jsize start, jsize len, jchar* data)
 {
+  jcharArray arr_local;
   BEGIN_EXCEPTION_HANDLING_VOID();
 
-  if (start >= obj_length((HArrayOfChar*)arr) || start+len > obj_length((HArrayOfChar*)arr)) {
+  arr_local = unveil(arr);
+  if (start >= obj_length((HArrayOfChar*)arr_local)
+      || start+len > obj_length((HArrayOfChar*)arr_local)) {
     throwException(ArrayIndexOutOfBoundsException);
   }
-  memcpy(&unhand_array((HArrayOfChar*)arr)->body[start], data, len * sizeof(jchar));
+  memcpy(&unhand_array((HArrayOfChar*)arr_local)->body[start], data,
+	 len * sizeof(jchar));
 
   END_EXCEPTION_HANDLING();
 }
@@ -637,12 +763,16 @@ KaffeJNI_SetCharArrayRegion(JNIEnv* env UNUSED, jcharArray arr, jsize start, jsi
 void
 KaffeJNI_SetShortArrayRegion(JNIEnv* env UNUSED, jshortArray arr, jsize start, jsize len, jshort* data)
 {
+  jshortArray arr_local;
   BEGIN_EXCEPTION_HANDLING_VOID();
 
-  if (start >= obj_length((HArrayOfShort*)arr) || start+len > obj_length((HArrayOfShort*)arr)) {
+  arr_local = unveil(arr);
+  if (start >= obj_length((HArrayOfShort*)arr_local)
+      || start+len > obj_length((HArrayOfShort*)arr_local)) {
     throwException(ArrayIndexOutOfBoundsException);
   }
-  memcpy(&unhand_array((HArrayOfShort*)arr)->body[start], data, len * sizeof(jshort));
+  memcpy(&unhand_array((HArrayOfShort*)arr_local)->body[start], data,
+	 len * sizeof(jshort));
 
   END_EXCEPTION_HANDLING();
 }
@@ -650,12 +780,16 @@ KaffeJNI_SetShortArrayRegion(JNIEnv* env UNUSED, jshortArray arr, jsize start, j
 void
 KaffeJNI_SetIntArrayRegion(JNIEnv* env UNUSED, jintArray arr, jsize start, jsize len, jint* data)
 {
+  jintArray arr_local;
   BEGIN_EXCEPTION_HANDLING_VOID();
 
-  if (start >= obj_length((HArrayOfInt*)arr) || start+len > obj_length((HArrayOfInt*)arr)) {
+  arr_local = unveil(arr);
+  if (start >= obj_length((HArrayOfInt*)arr_local)
+      || start+len > obj_length((HArrayOfInt*)arr_local)) {
     throwException(ArrayIndexOutOfBoundsException);
   }
-  memcpy(&unhand_array((HArrayOfInt*)arr)->body[start], data, len * sizeof(jint));
+  memcpy(&unhand_array((HArrayOfInt*)arr_local)->body[start], data,
+	 len * sizeof(jint));
 
   END_EXCEPTION_HANDLING();
 }
@@ -663,12 +797,16 @@ KaffeJNI_SetIntArrayRegion(JNIEnv* env UNUSED, jintArray arr, jsize start, jsize
 void
 KaffeJNI_SetLongArrayRegion(JNIEnv* env UNUSED, jlongArray arr, jsize start, jsize len, jlong* data)
 {
+  jlongArray arr_local;
   BEGIN_EXCEPTION_HANDLING_VOID();
 
-  if (start >= obj_length((HArrayOfLong*)arr) || start+len > obj_length((HArrayOfLong*)arr)) {
+  arr_local = unveil(arr);
+  if (start >= obj_length((HArrayOfLong*)arr_local)
+      || start+len > obj_length((HArrayOfLong*)arr_local)) {
     throwException(ArrayIndexOutOfBoundsException);
   }
-  memcpy(&unhand_array((HArrayOfLong*)arr)->body[start], data, len * sizeof(jlong));
+  memcpy(&unhand_array((HArrayOfLong*)arr_local)->body[start], data,
+	 len * sizeof(jlong));
 
   END_EXCEPTION_HANDLING();
 }
@@ -676,12 +814,16 @@ KaffeJNI_SetLongArrayRegion(JNIEnv* env UNUSED, jlongArray arr, jsize start, jsi
 void
 KaffeJNI_SetFloatArrayRegion(JNIEnv* env UNUSED, jfloatArray arr, jsize start, jsize len, jfloat* data)
 {
+  jfloatArray arr_local;
   BEGIN_EXCEPTION_HANDLING_VOID();
 
-  if (start >= obj_length((HArrayOfFloat*)arr) || start+len > obj_length((HArrayOfFloat*)arr)) {
+  arr_local = unveil(arr);
+  if (start >= obj_length((HArrayOfFloat*)arr_local)
+      || start+len > obj_length((HArrayOfFloat*)arr_local)) {
     throwException(ArrayIndexOutOfBoundsException);
   }
-  memcpy(&unhand_array((HArrayOfFloat*)arr)->body[start], data, len * sizeof(jfloat));
+  memcpy(&unhand_array((HArrayOfFloat*)arr_local)->body[start], data,
+	 len * sizeof(jfloat));
 
   END_EXCEPTION_HANDLING();
 }
@@ -689,12 +831,16 @@ KaffeJNI_SetFloatArrayRegion(JNIEnv* env UNUSED, jfloatArray arr, jsize start, j
 void
 KaffeJNI_SetDoubleArrayRegion(JNIEnv* env UNUSED, jdoubleArray arr, jsize start, jsize len, jdouble* data)
 {
+  jdoubleArray arr_local;
   BEGIN_EXCEPTION_HANDLING_VOID();
 
-  if (start >= obj_length((HArrayOfDouble*)arr) || start+len > obj_length((HArrayOfDouble*)arr)) {
+  arr_local = unveil(arr);
+  if (start >= obj_length((HArrayOfDouble*)arr_local)
+      || start+len > obj_length((HArrayOfDouble*)arr_local)) {
     throwException(ArrayIndexOutOfBoundsException);
   }
-  memcpy(&unhand_array((HArrayOfDouble*)arr)->body[start], data, len * sizeof(jdouble));
+  memcpy(&unhand_array((HArrayOfDouble*)arr_local)->body[start], data,
+	 len * sizeof(jdouble));
 
   END_EXCEPTION_HANDLING();
 }
