@@ -27,6 +27,7 @@
 #include "classMethod.h"
 #include "jvmpi_kaffe.h"
 #include "external.h"
+#include "system.h"
 
 /*
  * Keep track of how many VM's are active. Right now
@@ -267,12 +268,60 @@ KaffeJNI_ParseArgs(KaffeVM_Arguments *args, JavaVMOption *options, jint nOptions
   return 1;
 }
 
+static char *
+KaffeJNI_getSystemProperty(const char *key)
+{
+  jvalue retval;
+  Hjava_lang_String *key_string = stringC2Java(key);
+  
+  do_execute_java_class_method(&retval, "gnu/classpath/SystemProperties", NULL, "getProperty", "(Ljava/lang/String;)Ljava/lang/String;", key_string);
+  if (retval.l == NULL)
+    return NULL;
+
+  return stringJava2C(retval.l);
+}
+
+#if defined(ENABLE_JVMPI)
+
+static void
+detectAllActiveThreads(jthread_t jtid, void *param UNUSED)
+{
+  JVMPI_Event ev;
+  Hjava_lang_VMThread *tid;
+
+  tid = (Hjava_lang_VMThread *)(KTHREAD(get_data)(jtid)->jlThread);
+
+  jvmpiFillThreadStart(&ev, tid);
+  jvmpiPostEvent(&ev);
+  jvmpiCleanupThreadStart(&ev);
+}
+
+#endif /* defined(ENABLE_JVMPI) */
+
+static void KaffeJNI_addEndorsedDirs()
+{
+  char *endorsed = KaffeJNI_getSystemProperty("java.endorsed.dirs");
+  char *sep;
+  char *save_endorsed = endorsed;
+
+  if (endorsed == NULL)
+    return;
+
+  while ((sep = strstr(endorsed, path_separator)) != NULL)
+    {
+      *sep = 0;
+      KaffeVM_prependClasspath(endorsed);
+      endorsed = sep+1;
+    }
+
+  gc_free(save_endorsed);
+}
+
 jint
 JNI_CreateJavaVM(JavaVM** vm, void** penv, void* args)
 {
   JavaVMInitArgs *vm_args = (JavaVMInitArgs *)args;
   JNIEnv **env = (JNIEnv **)penv;
-  jnirefs *reftable;
 
   switch (vm_args->version)
     {
@@ -297,13 +346,7 @@ JNI_CreateJavaVM(JavaVM** vm, void** penv, void* args)
   /* Setup the machine */
   initialiseKaffe();
 
-  /* Setup JNI for main thread */
-  reftable =
-    (jnirefs *)gc_malloc(sizeof(jnirefs) + sizeof(jref) * DEFAULT_JNIREFS_NUMBER,
-			 KGC_ALLOC_STATIC_THREADDATA);
-  reftable->frameSize = DEFAULT_JNIREFS_NUMBER;
-  reftable->localFrames = 1;
-  THREAD_DATA()->jnireferences = reftable; 
+  KaffeJNI_addEndorsedDirs();
 
   /* Return the VM and JNI we're using */
   *vm = KaffeJNI_GetKaffeVM();
@@ -338,6 +381,16 @@ JNI_CreateJavaVM(JavaVM** vm, void** penv, void* args)
       ev.event_type = JVMPI_EVENT_JVM_INIT_DONE;
       jvmpiPostEvent(&ev);
     }
+
+ 
+  if ( JVMPI_EVENT_ISENABLED(JVMPI_EVENT_THREAD_START) )
+    {
+      KTHREAD(suspendall)();
+      
+      KTHREAD(walkLiveThreads) (detectAllActiveThreads, NULL);
+
+      KTHREAD(unsuspendall)();
+    } 
 #endif
 
   return 0;
@@ -349,6 +402,16 @@ KaffeJNI_DestroyJavaVM(JavaVM* vm UNUSED)
   /* The destroy function must be called by the same thread that has
    * built the VM object
    */
+#if defined(ENABLE_JVMPI)
+  if ( JVMPI_EVENT_ISENABLED(JVMPI_EVENT_JVM_SHUT_DOWN) )
+    {
+      JVMPI_Event ev;
+
+      ev.event_type = JVMPI_EVENT_JVM_SHUT_DOWN;
+      jvmpiPostEvent(&ev);
+    }
+#endif
+
   if (KTHREAD(current)() != startingThread)
     return -1;
 
