@@ -55,7 +55,7 @@ void *KaffeJNI_GetDirectBufferAddress(JNIEnv *env UNUSED, jobject buffer)
 {
   jobject buffer_obj = buffer;
   jobject address_obj;
-  void *address;
+  void *address = NULL;
   Hjava_lang_Object *buffer_local;
 
   BEGIN_EXCEPTION_HANDLING(NULL);
@@ -65,11 +65,11 @@ void *KaffeJNI_GetDirectBufferAddress(JNIEnv *env UNUSED, jobject buffer)
   if (!instanceof(javaNioBufferClass, OBJECT_CLASS(buffer_local)))
   {
 	  printf("buffer_local is %s\n", OBJECT_CLASS(buffer_local)->name->data);
-    address = NULL;
   } else
     {
       address_obj = KNI_GET_FIELD(Hjava_lang_Object *, buffer_local, directByteBufferImplAddress);
-      address = KNI_GET_FIELD(void *, address_obj, gnuClasspathPointerAddress);
+      if (NULL != address_obj)
+        address = KNI_GET_FIELD(void *, address_obj, gnuClasspathPointerAddress);
     }
 
   END_EXCEPTION_HANDLING();
@@ -116,7 +116,7 @@ KaffeJNI_FromReflectedMethod (JNIEnv *env UNUSED, jobject method)
   method_local = unveil(method);
   realMethod = (Hjava_lang_reflect_Method *)method_local;
 
-  id = (jmethodID) &(unhand(realMethod)->clazz->methods[unhand(realMethod)->slot]);
+  id = (jmethodID) &(Kaffe_get_class_methods(unhand(realMethod)->declaringClass)[unhand(realMethod)->slot]);
 
   END_EXCEPTION_HANDLING();
 
@@ -135,7 +135,7 @@ KaffeJNI_FromReflectedField (JNIEnv *env UNUSED, jobject field)
 
   realField = (Hjava_lang_reflect_Field *)field_local;
 
-  id = (jfieldID) &(unhand(realField)->clazz->fields[unhand(realField)->slot]);
+  id = (jfieldID) &(unhand(realField)->declaringClass->fields[unhand(realField)->slot]);
   END_EXCEPTION_HANDLING();
   
   return id;
@@ -155,16 +155,22 @@ KaffeJNI_ToReflectedMethod (JNIEnv *env UNUSED, jclass cls, jmethodID mid, jbool
 	cls_local = unveil(cls);
 	clazz = (Hjava_lang_Class *)cls_local;
 	refMeth = NULL;
-	for (allMethods = CLASS_METHODS(clazz), i = 0;
-	     i < CLASS_NMETHODS(clazz); 
-	     i++, allMethods++)
+	do
 	  {
-	    if (allMethods == (Method *)mid)
+	    for (allMethods = Kaffe_get_class_methods(clazz), i = 0;
+		 i < CLASS_NMETHODS(clazz); 
+		 i++, allMethods++)
 	      {
-		refMeth = KaffeVM_makeReflectMethod(clazz, i);
-		break;
+		if (allMethods == (Method *)mid)
+		  {
+		    refMeth = KaffeVM_makeReflectMethod(clazz, i);
+		    break;
+		  }
 	      }
+	    clazz = clazz->superclass;
 	  }
+	while (clazz != NULL && refMeth == NULL);
+
 	END_EXCEPTION_HANDLING();
 	
 	return (jobject) refMeth;
@@ -184,16 +190,21 @@ KaffeJNI_ToReflectedField (JNIEnv *env UNUSED, jclass cls, jfieldID fid, jboolea
 	cls_local = unveil(cls);
 	clazz = (Hjava_lang_Class *)cls_local;
 	refField = NULL;
-	for (allFields = CLASS_FIELDS(clazz), i = 0;
-	     i < CLASS_NFIELDS(clazz);
-	     i++, allFields++)
+	do
 	  {
-	    if (allFields == (Field *)fid)
+	    for (allFields = CLASS_FIELDS(clazz), i = 0;
+		 i < CLASS_NFIELDS(clazz);
+		 i++, allFields++)
 	      {
-		refField = KaffeVM_makeReflectField(clazz, i);
-		break;
+		if (allFields == (Field *)fid)
+		  {
+		    refField = KaffeVM_makeReflectField(clazz, i);
+		    break;
+		  }
 	      }
+	    clazz = clazz->superclass;
 	  }
+	while (clazz != NULL && refField == NULL);
 
 	END_EXCEPTION_HANDLING();
 	
