@@ -46,7 +46,6 @@ KaffeJNI_NewString(JNIEnv* env UNUSED, const jchar* data, jsize len)
   unhand(str)->offset = 0;
   unhand(str)->count = len;
   unhand(str)->value = (HArrayOfChar*)newArray(TYPE_CLASS(TYPE_Char), len);                   
-  unhand(str)->interned = 0;
   memcpy(STRING_DATA(str), data, len * sizeof(jchar));
 
   END_EXCEPTION_HANDLING();
@@ -101,7 +100,7 @@ KaffeJNI_NewStringUTF(JNIEnv* env UNUSED, const char* data)
     if (!utf8ConstIsValidUtf8(data, len)) {
       str = NULL;
     } else {
-      utf8 = checkPtr(utf8ConstNew(data, (int)len));
+      utf8 = checkPtr(utf8ConstNew(data, len));
       str = utf8Const2Java(utf8);
       utf8ConstRelease(utf8);
       if (!str) {
