@@ -92,30 +92,74 @@ static jclass
 Kaffe_DefineClass(JNIEnv* env, const char *name UNUSED, jobject loader,
 		  const jbyte* buf, jsize len)
 {
-	Hjava_lang_Class* cls;
-	classFile hand;
-	errorInfo info;
-	jobject loader_local;
+  Hjava_lang_Class* cls, *dup_cls;
+  classFile hand;
+  errorInfo info;
+  jobject loader_local;
+  classEntry *centry;
 
-	BEGIN_EXCEPTION_HANDLING(NULL);
+  BEGIN_EXCEPTION_HANDLING(NULL);
 
-	loader_local = unveil(loader); /* save clobbered reg.  */
+  loader_local = unveil(loader); /* save clobbered reg.  */
 
-	classFileInit(&hand, NULL, (unsigned char*)buf, (size_t)len,
-		      CP_BYTEARRAY);
+  classFileInit(&hand, NULL, (unsigned char*)buf, (size_t)len,
+		CP_BYTEARRAY);
 
-	cls = newClass();
-	if (cls == 0) {
-		postOutOfMemory(&info);
-	} else {
-		cls = readClass(cls, &hand, loader_local, &info);
+  cls = newClass();
+  if (cls == NULL) {
+    postOutOfMemory(&info);
+  } else {
+    cls = readClass(cls, &hand, loader_local, &info);
+  }
+  if (cls == NULL) {
+    postError(env, &info);
+  }
+
+  /*
+   * See if an entry for that name and class loader already exists
+   * create one if not.
+   */
+  centry = lookupClassEntry(cls->name, loader_local, &info);
+  if (centry == 0) {
+    throwError(&info);
+  }
+
+  if( classMappingLoad(centry, &dup_cls, &info) )
+    {
+      if(dup_cls != NULL)
+	{
+	  postExceptionMessage(&info,
+			       JAVA_LANG(ClassFormatError),
+			       "Duplicate name: %s",
+			       centry->name->data);
+	  throwError(&info);
 	}
-	if (cls == 0) {
-		postError(env, &info);
+      /*
+       * While it is not necessary that one be able to actually *use*
+       * the returned class object at this point, it is mandatory
+       * that the returned clazz object is a functional Class object.
+       *
+       * The following call will make sure that the returned class
+       * object has its dispatch table set.  The transition
+       * PRELOADED->PREPARED in processClass sets class->head.dtable.
+       *
+       * Presumably, it shouldn't be necessary here, but is at the
+       * moment - XXX
+       */
+      else if( processClass(cls,
+			    CSTATE_PREPARED,
+			    &info) == false )
+	{
+	  throwError(&info);
 	}
+    }
+  else
+    {
+      throwError(&info);
+    }
+  END_EXCEPTION_HANDLING();
 
-	END_EXCEPTION_HANDLING();
-	return (cls);
+  return cls;
 }
 
 static jclass
@@ -136,7 +180,7 @@ Kaffe_FindClass(JNIEnv *env UNUSED, const char* name)
 	classname2pathname (name, pathname);
 
 	/* create a new utf8 constant */
-	utf8 = utf8ConstNew(pathname, -1);
+	utf8 = utf8ConstFromString(pathname);
 
 	/* free the internal form of name */
 	KFREE(pathname);
@@ -563,7 +607,7 @@ Kaffe_GetFieldID(JNIEnv* env, jclass cls, const char* name, const char* sig UNUS
 	BEGIN_EXCEPTION_HANDLING(NULL);
 
 	cls_local = unveil(cls);
-	utf8 = checkPtr(utf8ConstNew(name, -1));
+	utf8 = checkPtr(utf8ConstFromString(name));
 	fld = lookupClassField((Hjava_lang_Class*)cls_local, utf8, false,
 			       &info);
 	utf8ConstRelease(utf8);
@@ -585,7 +629,7 @@ Kaffe_GetStaticMethodID(JNIEnv* env, jclass cls, const char* name, const char* s
 
 	cls_local = unveil(cls);
 	meth = lookupClassMethod((Hjava_lang_Class*)cls_local, name, sig,
-				 true, &info);
+				 false, &info);
 	if (meth == NULL) {
 		postError(env, &info);
 	} else if (!METHOD_IS_STATIC(meth)) {
@@ -609,7 +653,7 @@ Kaffe_GetStaticFieldID(JNIEnv* env, jclass cls, const char* name, const char* si
 	BEGIN_EXCEPTION_HANDLING(NULL);
 
 	cls_local = unveil(cls);
-	utf8 = checkPtr(utf8ConstNew(name, -1));
+	utf8 = checkPtr(utf8ConstFromString(name));
 	fld = lookupClassField((Hjava_lang_Class*)cls_local, utf8, true, &info);
 	utf8ConstRelease(utf8);
 	if (fld == NULL) {
@@ -646,7 +690,7 @@ Kaffe_RegisterNatives(JNIEnv* env UNUSED, jclass cls, const JNINativeMethod* met
 
 	cls_local = unveil(cls);
 
-	meth = CLASS_METHODS((Hjava_lang_Class*)cls_local);
+	meth = Kaffe_get_class_methods((Hjava_lang_Class*)cls_local);
 	nmeth = CLASS_NMETHODS((Hjava_lang_Class*)cls_local);
 
 	for (j = 0; j < nmethods; j++) {
@@ -654,7 +698,7 @@ Kaffe_RegisterNatives(JNIEnv* env UNUSED, jclass cls, const JNINativeMethod* met
 			if (strcmp(meth[i].name->data, methodArray[j].name) == 0 &&
 			    strcmp(METHOD_SIGD(&meth[i]), methodArray[j].signature) == 0 &&
 			    (meth[i].accflags & ACC_NATIVE) != 0) {
-				meth[i].accflags |= ACC_JNI;
+				meth[i].kFlags |= KFLAG_JNI;
 				engine_create_wrapper(&meth[i], methodArray[j].fnPtr); 
 				goto found;
 			}
@@ -728,8 +772,8 @@ Kaffe_AttachCurrentThread(JavaVM* vm UNUSED, void** penv, void* args UNUSED)
 {
 	if (KTHREAD(attach_current_thread) (false)) {
 		KSEM(init)(&THREAD_DATA()->sem);
-		KaffeVM_attachFakedThreadInstance ("test attach", false);
-		*penv = THREAD_JNIENV();
+		KaffeVM_attachFakedThreadInstance ("thread attach", false);
+		*penv = THREAD_JNIENV();		
 		return 0;
 	}
 	return -1;
@@ -750,7 +794,7 @@ Kaffe_AttachCurrentThreadAsDaemon(JavaVM* vm UNUSED, void** penv, void* args UNU
 static jint
 Kaffe_DetachCurrentThread(JavaVM* vm UNUSED)
 {
-	if (jthread_detach_current_thread ()) {
+	if (KTHREAD(detach_current_thread) ()) {
 		return 0;
 	} else {
 		return -1;
