@@ -34,6 +34,10 @@
 #include "gc-refs.h"
 #include "jvmpi_kaffe.h"
 
+#if defined(HAVE_SYS_TYPES_H)
+#include <sys/types.h>
+#endif /* defined(HAVE_SYS_TYPES_H) */
+
 static struct _gcStats {
         uint32  totalmem;
         uint32  totalobj;
@@ -247,7 +251,7 @@ gc_heap_isobject(gc_block *info, gc_unit *unit)
 		uint16 idx = GCMEM2IDX(info, unit);
 		if (idx < info->nr &&
 		    GCBLOCK2MEM(info, idx) == unit &&
-		    (KGC_GET_COLOUR(info, idx) & KGC_COLOUR_INUSE) == KGC_COLOUR_INUSE) {
+		    ((KGC_GET_COLOUR(info, idx) & KGC_COLOUR_INUSE) == KGC_COLOUR_INUSE || KGC_GET_COLOUR(info, idx) == KGC_COLOUR_FIXED)) {
 			return 1;
 		}
 	}
