@@ -58,16 +58,29 @@ getMethodSignatureClass(constIndex idx, Hjava_lang_Class* this, bool shouldLoadC
 	call->cname = NULL;
 
 	pool = CLASS_CONSTANTS(this);
+	if (idx >= CLASS_CONST_SIZE(this))
+	  {
+	    postExceptionMessage(einfo, JAVA_LANG(ClassFormatError), "invalid method name index");
+	    return false;
+	  }
+
 	if (pool->tags[idx] != CONSTANT_Methodref &&
 	    pool->tags[idx] != CONSTANT_InterfaceMethodref) {
 DBG(RESERROR,	dprintf("No Methodref found for idx=%d\n", idx);	);
 		/* shouldn't that be ClassFormatError or something? */
 		postExceptionMessage(einfo, JAVA_LANG(NoSuchMethodError),
-			"method name unknown, tag = %d", pool->tags[idx]);
+			"method name unknown, tag = %d",  pool->tags[idx]);
                 return (false);
 	}
 
 	ni = METHODREF_NAMEANDTYPE(idx, pool);
+	if (ni >= CLASS_CONST_SIZE(this))
+	  {
+	    postExceptionMessage(einfo, JAVA_LANG(ClassFormatError),
+				 "invalid method name index");
+	    return false;
+	  }
+
 	name = WORD2UTF(pool->data[NAMEANDTYPE_NAME(ni, pool)]);
 	sig = WORD2UTF(pool->data[NAMEANDTYPE_SIGNATURE(ni, pool)]);
 
@@ -158,6 +171,13 @@ getClass(constIndex idx, Hjava_lang_Class* this, errorInfo *einfo)
 
 	pool = CLASS_CONSTANTS(this);
 
+	if (idx >= CLASS_CONST_SIZE(this))
+	  {
+	    postExceptionMessage(einfo, JAVA_LANG(ClassFormatError),
+				 "invalid class name index");
+	    return NULL;
+	  }
+
 	tag = pool->tags[idx];
 
 	switch (tag) {
@@ -319,7 +339,7 @@ findMethodLocal(Hjava_lang_Class* class, Utf8Const* name, Utf8Const* signature)
 	 * Also there is no attempt to honour PUBLIC, PRIVATE, etc.
 	 */
 	n = CLASS_NMETHODS(class);
-	for (mptr = CLASS_METHODS(class); --n >= 0; ++mptr) {
+	for (mptr = Kaffe_get_class_methods(class); --n >= 0; ++mptr) {
 		if (utf8ConstEqual (name, mptr->name) && utf8ConstEqual (signature, METHOD_SIG(mptr))) {
 DBG(MLOOKUP,
 			dprintf("findMethodLocal(%s,%s,%s) -> %p\n",
