@@ -111,7 +111,7 @@ KaffeVM_makeReflectConstructor(struct Hjava_lang_Class* clazz, int slot)
 	Hjava_lang_reflect_Constructor* meth;
 	Method* mth;
 
-	mth = CLASS_METHODS(clazz) + slot;
+	mth = Kaffe_get_class_methods(clazz) + slot;
 	meth = (Hjava_lang_reflect_Constructor*)
 	    AllocObject("java/lang/reflect/Constructor", NULL);
 
@@ -129,11 +129,11 @@ KaffeVM_makeReflectMethod(struct Hjava_lang_Class* clazz, int slot)
 	Hjava_lang_reflect_Method* meth;
 	Method* mth;
 
-	mth = CLASS_METHODS(clazz) + slot;
+	mth = Kaffe_get_class_methods(clazz) + slot;
 	meth = (Hjava_lang_reflect_Method*)
 	    AllocObject("java/lang/reflect/Method", NULL);
 
-	unhand(meth)->clazz = clazz;
+	unhand(meth)->declaringClass = clazz;
 	unhand(meth)->slot = slot;
 	unhand(meth)->name = checkPtr(utf8Const2Java(mth->name));
 	unhand(meth)->parameterTypes = makeParameters(mth);
@@ -153,7 +153,7 @@ KaffeVM_makeReflectField(struct Hjava_lang_Class* clazz, int slot)
 	fld = CLASS_FIELDS(clazz) + slot;
 	field = (Hjava_lang_reflect_Field*)
 	    AllocObject("java/lang/reflect/Field", NULL);
-	unhand(field)->clazz = clazz;
+	unhand(field)->declaringClass = clazz;
 	unhand(field)->slot = slot;
 	unhand(field)->type = resolveFieldType(fld, clazz, &info);
 	if (unhand(field)->type == NULL) {
