@@ -1,7 +1,30 @@
+/*
+ * stats.c
+ * Getrusage based execution timing and other statistics gathering.
+ *
+ * Copyright (c) 1999
+ *	Archie Cobbs <archie@whistle.com>
+ *
+ * Copyright (c) 2004
+ *      Kaffe.org contributors. See ChangeLog for details. All rights reserved.
+ *   
+ * See the file "license.terms" for information on usage and redistribution 
+ * of this file. 
+ */
+
 #include "config.h"
 #include "config-std.h"
 #include "config-mem.h"
-#include "kaffe/jni_md.h"
+
+#if defined(HAVE_SYS_TIME_H)
+#include <sys/time.h>
+#endif
+
+#if defined(HAVE_SYS_RESOURCE_H)
+#include <sys/resource.h>
+#endif
+
+#include "jni_md.h"
 #if !defined(KAFFEH)
 #include "jthread.h"
 #endif
@@ -9,6 +32,9 @@
 #include "debug.h"
 #include "stats.h"
 
+#if defined(KAFFE_STATS)
+
+timespent fulljit;
 counter jitmem;
 counter jitcodeblock;
 counter utf8new;
@@ -19,8 +45,6 @@ counter jarmem;
 counter cpemem;
 timespent jit_time;
 
-#if defined(KAFFE_STATS)
-
 /*
  * The format for statMask (set by statsSetMaskStr) is like so:
  *	mask = element (',' element)*
