@@ -439,7 +439,9 @@ jthread_disable_stop(void)
 {
 	jthread_t tid = GET_JTHREAD();
 
-	tid->flags |= THREAD_FLAG_DONTSTOP;
+	/* XXX is called exactly once before initial thread context is created. */
+	if (tid != NULL)
+		tid->flags |= THREAD_FLAG_DONTSTOP;
 }
 
 /*
@@ -449,12 +451,16 @@ void
 jthread_enable_stop(void)
 {
 	jthread_t tid = GET_JTHREAD();
-
-	tid->flags &= ~THREAD_FLAG_DONTSTOP;
-	if (tid->status == THREAD_DYING) {
-		tid->status = THREAD_RUNNING;
-		onstop();
-		jthread_exit();
+	
+	/* XXX is called exactly once before initial thread context is created. */
+	if (tid != NULL)
+	{
+		tid->flags &= ~THREAD_FLAG_DONTSTOP;
+		if (tid->status == THREAD_DYING) {
+			tid->status = THREAD_RUNNING;
+			onstop();
+			jthread_exit();
+		}
 	}
 }
 
@@ -661,7 +667,7 @@ DBG(JTHREAD,
 		}
 
 		/* Am I suppose to close things down nicely ?? */
-		EXIT(0);
+		KAFFEVM_EXIT(0);
 	} else {
 		if (destructor1) {
 			(*destructor1)(tid->jlThread);
@@ -712,7 +718,7 @@ DBG(JTHREAD,
 void 
 jthread_dumpthreadinfo(jthread_t tid)
 {
-	fprintf(stderr, "jthread %p native %ld status %s\n", 
+	dprintf("jthread %p native %ld status %s\n", 
 		tid, (long) tid->native_thread,
 		tid->status == THREAD_NEWBORN ? "NEWBORN" :
 		tid->status == THREAD_RUNNING ? "RUNNING" :
