@@ -21,6 +21,10 @@
 #include "nets.h"
 #include "lock-impl.h"
 
+#if defined(HAVE_SYS_TYPES_H)
+#include <sys/types.h>
+#endif /* defined(HAVE_SYS_TYPES_H) */
+
 #if defined(HAVE_SYS_WAIT_H)
 #include <sys/wait.h>
 #endif
@@ -320,11 +324,11 @@ jthreadedRemove(const char *entry)
 
 static int     
 jthreadedSendto(int a, const void* b, size_t c, int d, const struct sockaddr* e,
-		int f, ssize_t *out)
+		socklen_t f, ssize_t *out)
 {
 	int rc = 0;
 
-	*out = e ? sendto(a, b, c, d, e, (socklen_t)f) : send(a, b, c, d);
+	*out = e ? sendto(a, b, c, d, e, f) : send(a, b, c, d);
 	if (*out == -1) {
 		rc = errno;
 	}
