@@ -4,7 +4,7 @@
  * Copyright (c) 1998
  *      Transvirtual Technologies, Inc.  All rights reserved.
  *
- * Copyright (c) 2005
+ * Copyright (c) 2005, 2006
  *      Kaffe.org contributors.  See ChangeLog for details. 
  *      All rights reserved.
  *
@@ -44,7 +44,7 @@
 #include "gc.h"
 #include "thread.h"
 #ifdef KAFFE_BOEHM_GC
-#include "boehm-gc/boehm/include/gc.h"
+#include <gc/gc.h>
 #endif
 
 /* define __USE_GNU for pthread_yield on linux */
@@ -68,19 +68,19 @@ static char stat_susp[]  = { ' ', 's', ' ', 'r', ' ', ' ', ' ', ' ', ' ', ' ', '
 static char stat_block[] = { ' ', 'T', 'm', ' ', 'c', ' ', ' ', ' ', 't', ' ', ' ' };
 
 #define TMSG_SHORT(_msg,_nt)     \
-   dprintf(_msg" %p [tid:%4lx, java:%p]\n", \
-    _nt, _nt->tid, _nt->data.jlThread)
+   dprintf(_msg" %p [java:%p]\n", \
+    _nt, _nt->data.jlThread)
 
 #define TMSG_LONG(_msg,_nt)      \
-   dprintf(_msg" %p [tid:%4lx, java:%p], stack [%p..%p..%p], state: %c%c%c\n",         \
-        _nt, _nt->tid, _nt->data.jlThread, _nt->stackMin, _nt->stackCur, _nt->stackMax,  \
+   dprintf(_msg" %p [java:%p], stack [%p..%p..%p], state: %c%c%c\n",         \
+        _nt, _nt->data.jlThread, _nt->stackMin, _nt->stackCur, _nt->stackMax,  \
         stat_act[_nt->active], stat_susp[_nt->suspendState], stat_block[_nt->blockState])
 
 #define CHECK_CURRENT_THREAD(_nt)                                          \
   if ( ((uintp) &_nt < (uintp) _nt->stackMin) ||           \
        ((uintp) &_nt > (uintp) _nt->stackMax) ) {          \
-    printf( "?? inconsistent current thread: %x [tid: %d, java: %x]\n",    \
-                    _nt, _nt->tid, _nt->data.jlThread);                                   \
+    printf( "?? inconsistent current thread: %x [java: %x]\n",    \
+                    _nt, _nt->data.jlThread);                                   \
     tDump();                                                               \
   }
 
@@ -96,7 +96,7 @@ static char stat_block[] = { ' ', 'T', 'm', ' ', 'c', ' ', ' ', ' ', 't', ' ', '
  * stringent priorities), but it usually isn't available on desktop
  * OSes (or imposes certain restrictions, e.g. root privileges).
  */
-#if defined(HAVE_SCHED_OTHER_IN_SCHED)
+#if defined(HAVE_SCHED_OTHER_IN_SCHED) && !(defined(__FreeBSD_kernel__) && defined(__GLIBC__))
 #define SCHEDULE_POLICY     SCHED_OTHER
 #else
 #undef SCHEDULE_POLICY
@@ -451,7 +451,7 @@ tInitSignals(void)
 
   if (SIGRTMAX - SIGRTMIN < 7)
     {
-#if defined(OLD_LINUXTHREADS)
+#if defined(OLD_LINUXTHREADS) && !defined(__CYGWIN__)
       sigSuspend = SIGURG;
       sigResume  = SIGTSTP;
       sigDump    = SIGXCPU;
@@ -474,8 +474,7 @@ tInitSignals(void)
 
 #endif
 
-      if (SIGRTMIN < 0)
-	sigInterrupt = SIGCONT;
+      sigInterrupt = SIGCONT;
     }
   else
     {
@@ -617,7 +616,7 @@ jthread_createfirst(size_t mainThreadStackSize, unsigned int pri UNUSED,
   /* Get stack boundaries. Note that this is just an approximation
    * which should cover all gc-relevant stack locations
    */
-  detectStackBoundaries(nt, mainThreadStackSize);
+  KaffePThread_detectStackBoundaries(nt, mainThreadStackSize);
 
   DBG( JTHREAD, TMSG_SHORT( "create first ", nt));
 
@@ -694,6 +693,9 @@ bool jthread_attach_current_thread (bool isDaemon)
   jthread_t		nt;
   rlim_t		stackSize;
 
+  if (jthread_current() != NULL)
+    return false;
+
   /* create the jthread* thingy */
   nt = thread_malloc( sizeof(struct _jthread) );
 
@@ -716,14 +718,16 @@ bool jthread_attach_current_thread (bool isDaemon)
 #else
   stackSize = MAINSTACKSIZE;
 #endif
-  detectStackBoundaries(nt, stackSize);
-  nt->stackCur     = NULL; 
-  nt->daemon       = isDaemon;
-
   /* link everything together */
   nt->tid = pthread_self();
   pthread_setspecific( ntKey, nt);
 
+  KaffePThread_detectThreadStackBoundaries(nt);
+  tInitSignalHandlers();
+
+  nt->stackCur     = NULL; 
+  nt->daemon       = isDaemon;
+
   /* and done */
   return true;
 }
@@ -1171,12 +1175,20 @@ jthread_destroy (jthread_t cur)
 void 
 jthread_sleep (jlong timeout) 
 {
+#ifdef __INTERIX
+	unsigned int seconds = timeout / 1000;
+	useconds_t   microseconds = (timeout % 1000) * 1000;
+
+	if (0 == sleep(seconds))
+		usleep(microseconds);
+#else
 	struct timespec ts;
 
 	ts.tv_sec  = timeout / 1000;
 	ts.tv_nsec = (timeout % 1000) * 1000000;
 
 	nanosleep (&ts, NULL);
+#endif
 }
 
 /***********************************************************************
@@ -1449,7 +1461,7 @@ jthread_suspendall (void)
 	 * Here, we must use the special Boehm's stop world routine.
 	 * However we continue to update our own thread state flag.
 	 */
-	GC_stop_world();
+	GC_disable();
 
 	for ( t=activeThreads; t; t = t->next ) {
 	  if ( (t != cur) && (t->suspendState == 0) && (t->active) ) {
@@ -1530,7 +1542,7 @@ jthread_unsuspendall (void)
 	  }
 	}
 
-	GC_start_world();
+	GC_enable();
 
 #endif
 	repsem_getvalue(&critSem, &val);
@@ -1776,8 +1788,16 @@ jthread_t jthread_current(void)
 {
   if (!jthreadInitialized)
     return NULL;
+  else {
+    void* specific = pthread_getspecific(ntKey);
 
-  return (jthread_t)pthread_getspecific(ntKey);
+    if (specific != NULL)
+      return (jthread_t) specific;
+    else {
+      perror(NULL);
+      abort();
+   }
+  }
 }
 
 /**
