@@ -15,7 +15,10 @@
 #ifndef __thread_internal_h
 #define __thread_internal_h
 
+#if !defined(__USE_GNU)
 #define __USE_GNU
+#endif /* !defined(__USE_GNU) */
+
 #include <pthread.h>
 #include "repsemaphore.h"
 
@@ -389,6 +392,7 @@ void KaffePThread_WaitForResume(int releaseMutex, unsigned int state);
 void KaffePThread_AckAndWaitForResume(jthread_t cur, unsigned int newState);
 int KaffePThread_getSuspendSignal(void);
 
-void detectStackBoundaries(jthread_t jtid, size_t mainThreadStackSize);
+void KaffePThread_detectStackBoundaries(jthread_t jtid, size_t mainThreadStackSize);
+void KaffePThread_detectThreadStackBoundaries(jthread_t jtid);
 
 #endif /* __thread_impl_h */
