@@ -283,7 +283,7 @@ verify3(Hjava_lang_Class* class, errorInfo *einfo)
 	});
 	
 	
-	for (n = CLASS_NMETHODS(class), method = CLASS_METHODS(class);
+	for (n = CLASS_NMETHODS(class), method = Kaffe_get_class_methods(class);
 	     n > 0;
 	     --n, ++method) {
 		
