@@ -13,6 +13,10 @@
  * University of Utah, http://www.cs.utah.edu/flux/
  */
 
+#include "config.h"
+
+#if defined(KAFFE_FEEDBACK)
+
 #include <stdio.h>
 #include <string.h>
 #include <stdarg.h>
@@ -21,8 +25,9 @@
 #include <stdlib.h>
 #include <sys/stat.h>
 
-#include "jmalloc.h"
+#include "kaffe/jmalloc.h"
 #include "stringSupport.h"
+#include "debug.h"
 
 #include "sectionFile.h"
 
@@ -59,7 +64,7 @@ struct parse_state {
 static int hashName(const char *name, int table_size)
 {
 	int h;
-	
+
 	for( h = 0; *name; name++ )
 		h = (64 * h + tolower(*name)) % table_size;
 	return( h );
@@ -71,7 +76,7 @@ static int hashName(const char *name, int table_size)
 static int validStringChar(int c)
 {
 	int retval = 0;
-	
+
 	if( isprint(c) &&
 	    (c != '#') &&
 	    (c != '%') &&
@@ -251,7 +256,7 @@ static int parseDirective(struct parse_state *ps, int curr)
 	line_len = ps->ps_len;
 	line = ps->ps_data;
 	save = ps->ps_save;
-	
+
 	/* The line passed in points to the beginning of the directive */
 	directive = &line[curr];
 	/* Move to the end of the directive */
@@ -278,7 +283,7 @@ static int parseDirective(struct parse_state *ps, int curr)
 			if( !syncFile(ps, ps->ps_section_file,
 				      args + name_start + 1) )
 			{
-				fprintf(stderr,
+				dprintf(
 					"Error:%s:%d:%d - Unable to process "
 					"file %s.\n",
 					ps->ps_filename,
@@ -290,7 +295,7 @@ static int parseDirective(struct parse_state *ps, int curr)
 		}
 		else
 		{
-			fprintf(stderr,
+			dprintf(
 				"Error:%s:%d:%d - %%include directive needs "
 				"a quoted filename.\n",
 				ps->ps_filename,
@@ -307,9 +312,9 @@ static int parseDirective(struct parse_state *ps, int curr)
 		char *section_type = 0, *section_name = 0;
 		struct file_section *type;
 		int stype;
-		
+
 		/* This is the beginning of a section */
-		
+
 		/* Parse the args into type and name */
 		ps->ps_data = args;
 		ps->ps_len = args_len;
@@ -321,7 +326,7 @@ static int parseDirective(struct parse_state *ps, int curr)
 		{
 		case SFS_INVALID:
 			retval = 0;
-			fprintf(stderr,
+			dprintf(
 				"Error:%s:%d - Invalid characters in section "
 				"name\n",
 				ps->ps_filename,
@@ -419,7 +424,7 @@ static int parseDirective(struct parse_state *ps, int curr)
 	}
 	else
 	{
-		fprintf(stderr,
+		dprintf(
 			"Error:%s:%d - Directive `%s' is not valid\n",
 			ps->ps_filename,
 			ps->ps_line,
@@ -447,7 +452,7 @@ static int parseFile(struct parse_state *ps)
 	in_file = ps->ps_in_file;
 	out_file = ps->ps_out_file;
 	ps->ps_data = buffer;
-	
+
 	/* Walk over the input file line by line */
 	while( retval && (line = fgets(buffer, MAX_LINE_SIZE, in_file)) )
 	{
@@ -527,7 +532,7 @@ static int parseFile(struct parse_state *ps)
 			}
 			else
 			{
-				fprintf(stderr,
+				dprintf(
 					"Error:%s:%d:%d - Text outside of "
 					"section\n",
 					ps->ps_filename,
@@ -565,7 +570,7 @@ static int writeNewSections(struct parse_state *ps)
 
 	sf = ps->ps_section_file;
 	out_file = ps->ps_out_file;
-	
+
 	/* Walk over the hash table links */
 	sfd = sf->sf_ordered_sections;
 	while( sfd && retval )
@@ -620,7 +625,7 @@ static int writeFile(struct parse_state *ps)
 
 	sf = ps->ps_section_file;
 	in_file = ps->ps_in_file;
-	
+
 	/* Create a temporary file for the output */
 	if( (temp_name = (char *)KMALLOC(10)) &&
 	    strcpy(temp_name, "sf.XXXXXX") &&
@@ -642,7 +647,7 @@ static int writeFile(struct parse_state *ps)
 		}
 		else
 		{
-			fprintf(stderr,
+			dprintf(
 				"Error: Unable to sync file %s\n",
 				ps->ps_filename);
 		}
@@ -663,7 +668,7 @@ static int writeFile(struct parse_state *ps)
 	}
 	else
 	{
-		fprintf(stderr,
+		dprintf(
 			"Error: Unable to create temporary file for "
 			"rewriting %s.\n",
 			ps->ps_filename);
@@ -820,7 +825,7 @@ int parseSectionLine(struct parse_state *ps, char **tag, char **value,
 	line = ps->ps_data;
 	line_len = ps->ps_len;
 	save = ps->ps_save;
-	
+
 	/* We start out looking for the tag */
 	state = SS_TAG;
 	/* Walk through the line */
@@ -1077,12 +1082,12 @@ char *makeFlagString(unsigned long flags, unsigned long flag_bit, char *value)
 
 int syncSectionFile(struct section_file *sf)
 {
-	int iLockRoot, retval;
+	int retval;
 
 	lockMutex(sf);
 	retval = syncFile(0, sf, sf->sf_filename);
 	unlockMutex(sf);
-	return( retval ); 
+	return( retval );
 }
 
 void addSectionType(struct file_section *fs)
@@ -1114,7 +1119,7 @@ struct file_section *findSectionType(char *name)
 
 void addSectionToFile(struct section_file *sf, struct section_file_data *sfd)
 {
-	int iLockRoot, hash;
+	int hash;
 
 	hash = hashName(sfd->sfd_name, SECTION_FILE_HASH_SIZE);
 	lockMutex(sf);
@@ -1138,7 +1143,7 @@ struct section_file_data *findSectionInFile(struct section_file *sf,
 					    char *name)
 {
 	struct section_file_data *retval = 0, *sfd;
-	int iLockRoot, hash;
+	int hash;
 
 	lockMutex(sf);
 	hash = hashName(name, SECTION_FILE_HASH_SIZE);
@@ -1161,7 +1166,7 @@ int walkFileSections(struct section_file *sf,
 		     void *arg)
 {
 	struct section_file_data *sfd;
-	int iLockRoot, retval = 1;
+	int retval = 1;
 
 	if( !sf )
 		return( 0 );
@@ -1185,7 +1190,7 @@ struct section_file_data *createFileSection(char *section_type,
 	struct file_section *fs;
 	char *new_name;
 	va_list args;
-	
+
 	va_start(args, section_name);
 	/* Get the section type and ask it to create a section data object */
 	if( (fs = findSectionType(section_type)) &&
@@ -1212,3 +1217,5 @@ void deleteFileSection(struct section_file_data *sfd)
 		sfd->sfd_type->fs_handler(sfd->sfd_type, 0, SFM_DELETE, sfd);
 	}
 }
+
+#endif /* KAFFE_FEEDBACK */
