@@ -4,7 +4,7 @@
  * Copyright (c) 1998
  *      Transvirtual Technologies, Inc.  All rights reserved.
  *
- * Copyright (c) 2002, 2003, 2004
+ * Copyright (c) 2002, 2003, 2004, 2005
  *	Kaffe.org contributors, see ChangeLog for details.  All rights reserved.
  *
  * See the file "license.terms" for information on usage and redistribution 
@@ -493,10 +493,10 @@ static inline void* getBuffer ( Toolkit* X, unsigned int nBytes ) {
  */
 void initColorMapping ( JNIEnv* env, jclass clazz, Toolkit* X);
 
-#define JRGB(_r,_g,_b)  (_r<<16 | _g<<8 | _b)
-#define JRED(_rgb)      ((_rgb & 0xff0000) >> 16)
-#define JGREEN(_rgb)    ((_rgb & 0x00ff00) >> 8)
-#define JBLUE(_rgb)     (_rgb & 0x0000ff)
+#define JRGB(_r,_g,_b)  (_r<<16 | _g<<8 | (_b))
+#define JRED(_rgb)      (((_rgb) & 0xff0000) >> 16)
+#define JGREEN(_rgb)    (((_rgb) & 0x00ff00) >> 8)
+#define JBLUE(_rgb)     ((_rgb) & 0x0000ff)
 
 
 /**
@@ -563,7 +563,7 @@ static inline int getFreeSourceIdx ( Toolkit* X, void* wnd ) {
    */
   for ( i = (unsigned long)wnd, n=0; n < X->nWindows; i++, n++ ) {
 	i %= X->nWindows;
-	if ( (int)(X->windows[i].w) <= 0 ) {
+	if ( (long)(X->windows[i].w) <= 0 ) {
 	  X->srcIdx = i;
 	  X->lastWindow = wnd;
 
