@@ -107,9 +107,9 @@ Java_java_math_BigInteger_assignLong0(JNIEnv* env, jobject r, jlong v)
 		   do-nothing operation.  At last, the number will be made
 		   negative, as appropriate.  */
 
-		mpz_set_ui(res, (unsigned long)((v & 0x7FFFFFFF00000000L) >> shift_distance));
+		mpz_set_ui(res, (unsigned long)((v & 0x7FFFFFFF00000000LL) >> shift_distance));
 		mpz_mul_2exp(res, res, shift_distance);
-		mpz_add_ui(res, res, (unsigned long)(v & 0x00000000FFFFFFFFL));
+		mpz_add_ui(res, res, (unsigned long)(v & 0x00000000FFFFFFFFLL));
 
 		if (negative)
 			mpz_neg(res, res);
@@ -159,13 +159,13 @@ jint
 Java_java_math_BigInteger_assignString0(JNIEnv* env, jobject r, jstring val, jint radix)
 {
 	mpz_ptr res;
-	const jbyte* str;
+	const char* str;
 	int rc;
 
 	res = (*env)->GetObjectField(env, r, number);
 	str = (*env)->GetStringUTFChars(env, val, NULL);
 
-	rc = mpz_set_str(res, (char*)str, (int)radix);
+	rc = mpz_set_str(res, str, (int)radix);
 
 	(*env)->ReleaseStringUTFChars(env, val, str);
 
