@@ -22,7 +22,9 @@ java_util_zip_Adler32_update(struct Hjava_util_zip_Adler32* this, HArrayOfByte*
 {
 	// XXX What happens if out of bounds ? 
 	if (from >= 0 && len > 0 && from + len <= obj_length(buf)) {
-		unhand(this)->adler = adler32((uint32)unhand(this)->adler, &unhand_array(buf)->body[from], (unsigned)len);
+		unhand(this)->adler = adler32((uLong)unhand(this)->adler, 
+					      (const Bytef *)&unhand_array(buf)->body[from], 
+					      (uInt)len);
 	}
 }
 
@@ -32,6 +34,8 @@ java_util_zip_Adler32_update1(struct Hjava_util_zip_Adler32* this, jint val)
 	jbyte b;
 
 	b = val;
-	unhand(this)->adler = adler32((uint32)unhand(this)->adler, &b, sizeof(b));
+	unhand(this)->adler = adler32((uLong)unhand(this)->adler, 
+				      (const Bytef *)&b, 
+				      (uInt)sizeof(b));
 }
 
