@@ -22,6 +22,8 @@ Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 
 package gnu.classpath.examples.swing;
 
+import gnu.classpath.examples.java2d.JNIOverhead;
+
 import java.awt.*;
 import java.awt.event.*;
 
@@ -33,6 +35,7 @@ import javax.swing.plaf.metal.MetalLookAndFeel;
 import javax.swing.plaf.metal.MetalTheme;
 import javax.swing.plaf.metal.OceanTheme;
 
+import java.lang.reflect.Method;
 import java.net.URL;
 
 public class Demo
@@ -150,6 +153,15 @@ public class Demo
     examples.add(new JMenuItem(new PopupAction("Theme Editor",
                                        MetalThemeEditor.createDemoFactory())));
 
+    examples.add(new JMenuItem(new PopupAction("DocumentFilter",
+                                     DocumentFilterDemo.createDemoFactory())));
+
+    examples.add(new JMenuItem(new PopupAction("NavigationFilter",
+                                               NavigationFilterDemo.createDemoFactory())));
+    examples.add(new JMenuItem(new PopupAction("JNI Overhead",
+                                               JNIOverhead.createDemoFactory())));
+
+
     final JMenuItem vmMenu;
     
     help.add(new JMenuItem("just play with the widgets"));
@@ -166,7 +178,8 @@ public class Demo
                              + " Version "
                              + System.getProperty("java.vm.version")
                              + " distributed by "
-                             + System.getProperty("java.vm.vendor");
+                             + System.getProperty("java.vm.vendor")
+                             + ".";
                          
             String gnuClasspath = System.getProperty("gnu.classpath.version");
             if(gnuClasspath != null)
@@ -192,22 +205,46 @@ public class Demo
         boolean selected = laf.getClassName().equals(currentLaf);
         lafItem.setSelected(selected);
         lafMenu.add(lafItem);
+        
+        lafGroup.add(lafItem);
       }
 
     // Create themes menu.
     themesMenu = new JMenu("Themes");
     ButtonGroup themesGroup = new ButtonGroup();
-    JRadioButtonMenuItem ocean =
-      new JRadioButtonMenuItem(new ChangeThemeAction(new OceanTheme()));
-    ocean.setSelected(MetalLookAndFeel.getCurrentTheme() instanceof OceanTheme);
-    themesMenu.add(ocean);
-    themesGroup.add(ocean);
-    JRadioButtonMenuItem steel =
-      new JRadioButtonMenuItem(new ChangeThemeAction(new DefaultMetalTheme()));
-    ocean.setSelected(MetalLookAndFeel.getCurrentTheme()
-                      instanceof DefaultMetalTheme);
-    themesMenu.add(steel);
-    themesGroup.add(steel);
+
+    // In order to make the demo runable on a 1.4 type VM we have to avoid calling
+    // MetalLookAndFeel.getCurrentTheme(). We simply check whether this method exists
+    // and is public.
+    Method m = null;
+    try
+      {
+        m = MetalLookAndFeel.class.getMethod("getCurrentTheme", null);
+      }
+    catch (NoSuchMethodException nsme)
+      {
+        // Ignore it.
+      }
+    
+    if (m != null)
+      {
+        JRadioButtonMenuItem ocean =
+          new JRadioButtonMenuItem(new ChangeThemeAction(new OceanTheme()));
+        ocean.setSelected(MetalLookAndFeel.getCurrentTheme() instanceof OceanTheme);
+        themesMenu.add(ocean);
+        themesGroup.add(ocean);
+    
+        JRadioButtonMenuItem steel =
+          new JRadioButtonMenuItem(new ChangeThemeAction(new DefaultMetalTheme()));
+        ocean.setSelected(MetalLookAndFeel.getCurrentTheme()
+                          instanceof DefaultMetalTheme);
+        themesMenu.add(steel);
+        themesGroup.add(steel);
+      }
+    else
+      {
+        themesMenu.setEnabled(false);
+      }
     
     bar.add(file);
     bar.add(edit);
@@ -334,7 +371,10 @@ public class Demo
     JPanel main = new JPanel();
     main.setLayout(new BoxLayout(main, BoxLayout.Y_AXIS));
     desktop = createDesktop();
-    main.add(desktop);
+    
+    // Put the desktop in a scrollpane. The scrollbars may show then
+    // up when the them or LaF is changed.
+    main.add(new JScrollPane(desktop));
     main.add(mkButtonBar());
     component.add(main, BorderLayout.CENTER);
     frame.pack();
@@ -510,7 +550,10 @@ public class Demo
     panel.add(new JButton(new PopupAction("Tree",
                                           TreeDemo.createDemoFactory())));
     panel.add(new JButton(new PopupAction("Theme Editor",
-                                       MetalThemeEditor.createDemoFactory())));
+                                      MetalThemeEditor.createDemoFactory())));
+    panel.add(new JButton(new PopupAction("JNI Overhead",
+                                          JNIOverhead.createDemoFactory())));
+
     JButton exitDisposer = mkDisposerButton(frame);
     panel.add(exitDisposer);
     
