@@ -67,7 +67,7 @@ public class PrintStream extends FilterOutputStream
 
   // Line separator string.
   private static final char[] line_separator
-    = SystemProperties.getProperty("line.separator").toCharArray();
+    = SystemProperties.getProperty("line.separator", "\n").toCharArray();
 
   /**
    *  Encoding name
@@ -86,11 +86,77 @@ public class PrintStream extends FilterOutputStream
    */
   private boolean auto_flush;
 
-  private boolean defaultEncoding;
+  /**
+   * This method initializes a new <code>PrintStream</code> object to write
+   * to the specified output File. Doesn't autoflush.
+   *
+   * @param file The <code>File</code> to write to.
+   * @throws FileNotFoundException if an error occurs while opening the file.
+   *
+   * @since 1.5
+   */
+  public PrintStream (File file)
+    throws FileNotFoundException
+  {
+    this (new FileOutputStream(file), false);
+  }
 
   /**
-   * This method intializes a new <code>PrintStream</code> object to write
-   * to the specified output sink.
+   * This method initializes a new <code>PrintStream</code> object to write
+   * to the specified output File. Doesn't autoflush.
+   *
+   * @param file The <code>File</code> to write to.
+   * @param encoding The name of the character encoding to use for this
+   * object.
+   * @throws FileNotFoundException If an error occurs while opening the file.
+   * @throws UnsupportedEncodingException If the charset specified by
+   * <code>encoding</code> is invalid.
+   *
+   * @since 1.5
+   */
+  public PrintStream (File file, String encoding)
+    throws FileNotFoundException,UnsupportedEncodingException
+  {
+    this (new FileOutputStream(file), false, encoding);
+  }
+
+  /**
+   * This method initializes a new <code>PrintStream</code> object to write
+   * to the specified output File. Doesn't autoflush.
+   *
+   * @param fileName The name of the <code>File</code> to write to.
+   * @throws FileNotFoundException if an error occurs while opening the file,
+   *
+   * @since 1.5
+   */
+  public PrintStream (String fileName)
+    throws FileNotFoundException
+  {
+    this (new FileOutputStream(new File(fileName)), false);
+  }
+
+  /**
+   * This method initializes a new <code>PrintStream</code> object to write
+   * to the specified output File. Doesn't autoflush.
+   *
+   * @param fileName The name of the <code>File</code> to write to.
+   * @param encoding The name of the character encoding to use for this
+   * object.
+   * @throws FileNotFoundException if an error occurs while opening the file.
+   * @throws UnsupportedEncodingException If the charset specified by
+   * <code>encoding</code> is invalid.
+   *
+   * @since 1.5
+   */
+  public PrintStream (String fileName, String encoding)
+      throws FileNotFoundException,UnsupportedEncodingException
+  {
+    this (new FileOutputStream(new File(fileName)), false, encoding);
+  }
+
+  /**
+   * This method initializes a new <code>PrintStream</code> object to write
+   * to the specified output sink. Doesn't autoflush.
    *
    * @param out The <code>OutputStream</code> to write to.
    */
@@ -100,7 +166,7 @@ public class PrintStream extends FilterOutputStream
   }
 
   /**
-   * This method intializes a new <code>PrintStream</code> object to write
+   * This method initializes a new <code>PrintStream</code> object to write
    * to the specified output sink.  This constructor also allows "auto-flush"
    * functionality to be specified where the stream will be flushed after
    * every <code>print</code> or <code>println</code> call, when the 
@@ -126,11 +192,10 @@ public class PrintStream extends FilterOutputStream
 	this.encoding = "ISO8859_1";
     }
     this.auto_flush = auto_flush;
-    this.defaultEncoding = true;
   }
 
   /**
-   * This method intializes a new <code>PrintStream</code> object to write
+   * This method initializes a new <code>PrintStream</code> object to write
    * to the specified output sink.  This constructor also allows "auto-flush"
    * functionality to be specified where the stream will be flushed after
    * every <code>print</code> or <code>println</code> call, when the 
@@ -262,43 +327,15 @@ public class PrintStream extends FilterOutputStream
   private void writeChars(char[] buf, int offset, int count)
     throws IOException
   {
-    try
-      {
-	byte[] bytes = (new String(buf, offset, count)).getBytes(encoding);
-	out.write(bytes, 0, bytes.length);
-      }
-    catch (UnsupportedEncodingException e)
-      {
-	if (defaultEncoding)
-	  {
-	    this.encoding = "ISO8859_1";
-	    writeChars(buf, offset, count);
-	  }
-	else
-	  throw e;
-      }
+      byte[] bytes = (new String(buf, offset, count)).getBytes(encoding);
+      out.write(bytes, 0, bytes.length);
   }
 
   private void writeChars(String str, int offset, int count)
     throws IOException
   {
-    try
-      {
-	byte[] bytes = str.substring(offset, offset+count).getBytes(encoding);
-	out.write(bytes, 0, bytes.length);
-      }
-    catch (UnsupportedEncodingException e)
-      {
-	if (defaultEncoding)
-	  {
-	    this.encoding = "ISO8859_1";
-	    writeChars(str, offset, count);
-	  }
-	else
-	  {
-	    throw e;
-	  }
-      }
+      byte[] bytes = str.substring(offset, offset+count).getBytes(encoding);
+      out.write(bytes, 0, bytes.length);
   }
 
   /**
