@@ -47,6 +47,7 @@ import javax.swing.JComponent;
 import javax.swing.JTabbedPane;
 import javax.swing.UIManager;
 import javax.swing.plaf.ComponentUI;
+import javax.swing.plaf.UIResource;
 import javax.swing.plaf.basic.BasicTabbedPaneUI;
 
 /**
@@ -136,7 +137,12 @@ public class MetalTabbedPaneUI extends BasicTabbedPaneUI
 
   /** The graphics to draw the highlight below the tab. */
   private Graphics hg;
-  
+
+  /**
+   * Indicates if the tabs are having their background filled.
+   */
+  private boolean tabsOpaque;
+
   /**
    * Constructs a new instance of MetalTabbedPaneUI.
    */
@@ -326,13 +332,13 @@ public class MetalTabbedPaneUI extends BasicTabbedPaneUI
     int firstIndex = tabRuns[currentRun];
 
     // Paint the part of the above tab.
-    if (tabIndex != firstIndex)
+    if (tabIndex != firstIndex && tabIndex > 0 && tabsOpaque)
       {
         Color c;
         if (tabPane.getSelectedIndex() == tabIndex - 1)
           c = selectColor;
         else
-          c = UIManager.getColor("TabbedPane.unselectedBackground");
+          c = getUnselectedBackground(tabIndex - 1);
         g.setColor(c);
         g.fillRect(2, 0, 4, 3);
         g.drawLine(2, 3, 2, 3);
@@ -453,13 +459,13 @@ public class MetalTabbedPaneUI extends BasicTabbedPaneUI
     int firstIndex = tabRuns[currentRun];
 
     // Paint part of the above tab.
-    if (tabIndex != firstIndex)
+    if (tabIndex != firstIndex && tabIndex > 0 && tabsOpaque)
       {
         Color c;
         if (tabPane.getSelectedIndex() == tabIndex - 1)
           c = UIManager.getColor("TabbedPane.tabAreaBackground");
         else
-          c = UIManager.getColor("TabbedPane.unselectedBackground");
+          c = getUnselectedBackground(tabIndex - 1);
         g.fillRect(right - 5, 0, 5, 3);
         g.fillRect(right - 2, 3, 2, 2);
       }
@@ -601,11 +607,11 @@ public class MetalTabbedPaneUI extends BasicTabbedPaneUI
       }
     else
       {
-        if (isOcean && tabIndex == tabPane.getSelectedIndex()+ 1)
+        if (isOcean && tabIndex == tabPane.getSelectedIndex() + 1)
           {
             g.setColor(oceanSelectedBorder);
           }
-        if (tabIndex != tabRuns[runCount- 1])
+        if (tabIndex != tabRuns[runCount - 1])
           {
             g.drawLine(0, 0, 0, bottom);
           }
@@ -658,7 +664,7 @@ public class MetalTabbedPaneUI extends BasicTabbedPaneUI
     if (isSelected)
       g.setColor(selectColor);
     else
-      g.setColor(tabPane.getBackgroundAt(tabIndex));
+      g.setColor(getUnselectedBackground(tabIndex));
 
     switch (tabPlacement)
     {
@@ -708,6 +714,7 @@ public class MetalTabbedPaneUI extends BasicTabbedPaneUI
     selectColor = UIManager.getColor("TabbedPane.selected");
     selectHighlight = UIManager.getColor("TabbedPane.selectHighlight");
     tabAreaBackground = UIManager.getColor("TabbedPane.tabAreaBackground");
+    tabsOpaque = UIManager.getBoolean("TabbedPane.tabsOpaque");
     minTabWidth = 0;
   }
   
@@ -849,7 +856,7 @@ public class MetalTabbedPaneUI extends BasicTabbedPaneUI
     // run directly above the content or the selected tab is not visible,
     // then we draw an unbroken line.
     if (tabPlacement != TOP || selectedIndex < 0
-        || rect.y  + rect.height + 1 < y || rect.x < x ||rect.x > x + w)
+        || rect.y  + rect.height + 1 < y || rect.x < x || rect.x > x + w)
       {
         g.drawLine(x, y, x + w - 2, y);
         if (isOcean && tabPlacement == TOP)
@@ -1115,4 +1122,26 @@ public class MetalTabbedPaneUI extends BasicTabbedPaneUI
     int lastIndex = lastTabInRun(count, run);
     return tabIndex == lastIndex;
   }
+
+  /**
+   * Returns the background for an unselected tab. This first asks the
+   * JTabbedPane for the background at the specified tab index, if this
+   * is an UIResource (that means, it is inherited from the JTabbedPane)
+   * and the TabbedPane.unselectedBackground UI property is not null,
+   * this returns the value of the TabbedPane.unselectedBackground property,
+   * otherwise the value returned by the JTabbedPane.
+   *
+   * @param tabIndex the index of the tab for which we query the background
+   *
+   * @return the background for an unselected tab
+   */
+  private Color getUnselectedBackground(int tabIndex)
+  {
+    Color bg = tabPane.getBackgroundAt(tabIndex);
+    Color unselectedBackground =
+      UIManager.getColor("TabbedPane.unselectedBackground");
+    if (bg instanceof UIResource && unselectedBackground != null)
+      bg = unselectedBackground;
+    return bg;
+  }
 }
