@@ -80,7 +80,6 @@ static GtkWidget *get_widget (GtkWidget *widget);
 
 static jmethodID postMouseEventID;
 static jmethodID postMouseWheelEventID;
-static jmethodID setCursorID;
 static jmethodID postExposeEventID;
 static jmethodID postFocusEventID;
 
@@ -100,9 +99,6 @@ cp_gtk_component_init_jni (void)
 						        "postMouseWheelEvent",
 							"(IJIIIIZIII)V");
 
-  setCursorID = (*cp_gtk_gdk_env())->GetMethodID (cp_gtk_gdk_env(), gtkcomponentpeer,
-                                           "setCursor", "()V");
-
   postExposeEventID = (*cp_gtk_gdk_env())->GetMethodID (cp_gtk_gdk_env(), gtkcomponentpeer,
                                                  "postExposeEvent", "(IIII)V");
 
@@ -192,22 +188,23 @@ state_to_awt_mods_with_button_states (guint state)
 
 JNIEXPORT void JNICALL 
 Java_gnu_java_awt_peer_gtk_GtkComponentPeer_gtkWidgetSetCursor 
-  (JNIEnv *env, jobject obj, jint type) 
+  (JNIEnv *env, jobject obj, jint type, jobject image, jint x, jint y) 
 {
   gdk_threads_enter ();
 
   Java_gnu_java_awt_peer_gtk_GtkComponentPeer_gtkWidgetSetCursorUnlocked
-    (env, obj, type);
+    (env, obj, type, image, x, y);
 
   gdk_threads_leave ();
 }
 
 JNIEXPORT void JNICALL 
 Java_gnu_java_awt_peer_gtk_GtkComponentPeer_gtkWidgetSetCursorUnlocked
-  (JNIEnv *env, jobject obj, jint type) 
+  (JNIEnv *env, jobject obj, jint type, jobject image, jint x, jint y) 
 {
   void *ptr;
   GtkWidget *widget;
+  GdkWindow *win;
   GdkCursorType gdk_cursor_type;
   GdkCursor *gdk_cursor;
 
@@ -259,10 +256,24 @@ Java_gnu_java_awt_peer_gtk_GtkComponentPeer_gtkWidgetSetCursorUnlocked
     }
       
   widget = get_widget(GTK_WIDGET(ptr));
+  
+  win = widget->window;
+  if ((widget->window) == NULL)
+    win = GTK_WIDGET(ptr)->window;
+    
+  if (image == NULL)
+    gdk_cursor = gdk_cursor_new (gdk_cursor_type);
+  else
+    gdk_cursor
+      = gdk_cursor_new_from_pixbuf (gdk_drawable_get_display (win),
+				    cp_gtk_image_get_pixbuf (env, image),
+				    x, y);
 
-  gdk_cursor = gdk_cursor_new (gdk_cursor_type);
-  gdk_window_set_cursor (widget->window, gdk_cursor);
+  gdk_window_set_cursor (win, gdk_cursor);
   gdk_cursor_unref (gdk_cursor);
+
+  /* Make sure the cursor is replaced on screen. */
+  gdk_flush();
 }
 
 JNIEXPORT void JNICALL
@@ -742,6 +753,20 @@ Java_gnu_java_awt_peer_gtk_GtkComponentPeer_gtkWidgetSetForeground
   gdk_threads_leave ();
 }
 
+JNIEXPORT void JNICALL
+Java_gnu_java_awt_peer_gtk_GtkComponentPeer_realize (JNIEnv *env, jobject obj)
+{
+  void *ptr;
+
+  gdk_threads_enter ();
+
+  ptr = NSA_GET_PTR (env, obj);
+
+  gtk_widget_realize (GTK_WIDGET (ptr));
+
+  gdk_threads_leave ();
+}
+
 JNIEXPORT void JNICALL
 Java_gnu_java_awt_peer_gtk_GtkComponentPeer_setVisibleNative
   (JNIEnv *env, jobject obj, jboolean visible)
@@ -786,30 +811,6 @@ Java_gnu_java_awt_peer_gtk_GtkComponentPeer_isEnabled
   return ret_val;
 }
 
-JNIEXPORT jboolean JNICALL 
-Java_gnu_java_awt_peer_gtk_GtkComponentPeer_isRealized
-  (JNIEnv *env, jobject obj)
-{
-  void *ptr;
-  jboolean ret_val;
-
-  gdk_threads_enter ();
-
-  ptr = NSA_GET_PTR (env, obj);
-
-  if (ptr == NULL)
-    {
-      gdk_threads_leave ();
-      return FALSE;
-    }
-
-  ret_val = GTK_WIDGET_REALIZED (get_widget(GTK_WIDGET (ptr)));
-
-  gdk_threads_leave ();
-
-  return ret_val;
-}
-
 JNIEXPORT jboolean JNICALL 
 Java_gnu_java_awt_peer_gtk_GtkComponentPeer_modalHasGrab
   (JNIEnv *env __attribute__((unused)), jclass clazz __attribute__((unused)))
