@@ -3,8 +3,9 @@ class btest extends Thread
 
   public void run()
     {
-      System.out.println(isAlive()+" 1");
-      this.stop();
+      synchronized(this) {
+	System.out.println(isAlive()+" 1");
+      }
     }
 }
 
@@ -16,11 +17,18 @@ public class DeadThread
       int c = 0;
         
       b = new btest();
-      b.start();
-      while(b.isAlive() && c < 10)
+      synchronized(b) {
+        b.start();
+        while(b.isAlive() && c < 10)
         {
           System.out.println(++c+" "+b.isAlive());
-          try{Thread.sleep(1000);}catch(InterruptedException e){}
+          try{b.wait(1000);}catch(InterruptedException e){}
         }
+      }
     }
 }
+
+/* Expected Output:
+1 true
+true 1
+*/
