@@ -1,5 +1,4 @@
 
-import org.kaffe.lang.PackageHelper;
 import gnu.classpath.SystemProperties;
 
 /**
@@ -9,15 +8,6 @@ class KaffeInternal
 {
     public static void main(String args[])
     {
-	try
-	{
-            PackageHelper.getPackageName (KaffeInternal.class);
-	}
-	catch(Throwable th)
-	{
-	    System.out.println(th);
-	}
-
         try
         {
             SystemProperties.getProperty("java.home");
@@ -30,6 +20,5 @@ class KaffeInternal
 }
 
 /* Expected Output:
-java.lang.NoClassDefFoundError: org/kaffe/lang/PackageHelper
 java.lang.NoClassDefFoundError: gnu/classpath/SystemProperties
 */
