@@ -3,8 +3,7 @@ class btst1 extends Thread
 
   public void run()
     {
-      System.err.println(isAlive()+" 1");
-      this.stop();
+      System.out.println(isAlive()+" 1");
     }
 }
 
@@ -25,3 +24,9 @@ public class tthrd1
       System.err.println("main end");
     }
 }
+
+/* Expected Output:
+main start
+true 1
+main end
+*/
